@@ -6,7 +6,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     8/7/2000
-// RCS-ID:      $Id: splittree.cpp 35650 2005-09-23 12:56:45Z MR $
+// RCS-ID:      $Id: splittree.cpp 51581 2008-02-07 16:42:22Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -38,6 +38,7 @@
 #endif
 
 #include "wx/gizmos/splittree.h"
+#include "wx/renderer.h"
 #include <math.h>
 
 /*
@@ -98,6 +99,39 @@ void wxRemotelyScrolledTreeCtrl::HideVScrollbar()
 #endif
 }
 
+void wxRemotelyScrolledTreeCtrl::DoCalcScrolledPosition(int x, int y, int *xx, int *yy) const
+{
+#if USE_GENERIC_TREECTRL || !defined(__WXMSW__)
+    if (IsKindOf(CLASSINFO(wxGenericTreeCtrl)))
+    {
+        wxGenericTreeCtrl* win = (wxGenericTreeCtrl*) this;
+        * yy = 0;
+        int yyy;
+        win->wxGenericTreeCtrl::DoCalcScrolledPosition(x, y, xx, & yyy);
+
+        wxScrolledWindow* scrolledWindow = GetScrolledWindow();
+        if (scrolledWindow)
+        {
+            int xxx;
+            scrolledWindow->DoCalcScrolledPosition(x, y, & xxx, yy);
+        }
+    }
+#endif
+}
+
+void wxRemotelyScrolledTreeCtrl::SetScrollbar(int orient,
+                               int pos,
+                               int thumbVisible,
+                               int range,
+                               bool update)
+{
+#ifndef __WXMSW__
+    if (orient == wxVERTICAL)
+        range = 0;
+#endif
+    wxWindow::SetScrollbar(orient, pos, thumbVisible, range, update);
+}
+
 // Number of pixels per user unit (0 or -1 for no scrollbar)
 // Length of virtual canvas in user units
 // Length of page in user units
@@ -119,7 +153,7 @@ void wxRemotelyScrolledTreeCtrl::SetScrollbars(
     if (IsKindOf(CLASSINFO(wxGenericTreeCtrl)))
     {
         wxGenericTreeCtrl* win = (wxGenericTreeCtrl*) this;
-        win->wxGenericTreeCtrl::SetScrollbars(pixelsPerUnitX, pixelsPerUnitY, noUnitsX, 0, xPos, 0, /* noRefresh */ true);
+        win->wxGenericTreeCtrl::SetScrollbars(pixelsPerUnitX, pixelsPerUnitY, noUnitsX, noUnitsY, xPos, yPos, /* noRefresh */ true);
 
         wxScrolledWindow* scrolledWindow = GetScrolledWindow();
         if (scrolledWindow)
@@ -447,6 +481,10 @@ void wxRemotelyScrolledTreeCtrl::OnScroll(wxScrollWinEvent& event)
     scrollWin->GetViewStart(& x, & y);
 
     ScrollToLine(-1, y);
+
+#ifndef __WXMSW__
+    m_yScrollPosition = GetScrollPos(wxVERTICAL);
+#endif
 }
 
 /*
@@ -602,6 +640,12 @@ bool wxThinSplitterWindow::SashHitTest(int x, int y, int WXUNUSED(tolerance))
 
 void wxThinSplitterWindow::DrawSash(wxDC& dc)
 {
+    wxRendererNative::Get().DrawSplitterBorder
+                           (
+                               this,
+                               dc,
+                               GetClientRect()
+                           );
     if ( m_sashPosition == 0 || !m_windowTwo)
         return;
     if (GetWindowStyle() & wxSP_NOSASH)
