@@ -3,7 +3,7 @@
 // Purpose:     wx wrappers for DirectFB interfaces
 // Author:      Vaclav Slavik
 // Created:     2006-08-23
-// RCS-ID:      $Id: wrapdfb.h 43729 2006-12-01 13:02:20Z VS $
+// RCS-ID:      $Id: wrapdfb.h 53110 2008-04-10 17:58:44Z VS $
 // Copyright:   (c) 2006 REA Elektronik GmbH
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -208,10 +208,10 @@ struct wxIDirectFBSurface : public wxDfbWrapper<IDirectFBSurface>
     bool SetClip(const DFBRegion *clip)
         { return Check(m_ptr->SetClip(m_ptr, clip)); }
 
-    bool SetColor(__u8 r, __u8 g, __u8 b, __u8 a)
+    bool SetColor(u8 r, u8 g, u8 b, u8 a)
         { return Check(m_ptr->SetColor(m_ptr, r, g, b, a)); }
 
-    bool Clear(__u8 r, __u8 g, __u8 b, __u8 a)
+    bool Clear(u8 r, u8 g, u8 b, u8 a)
         { return Check(m_ptr->Clear(m_ptr, r, g, b, a)); }
 
     bool DrawLine(int x1, int y1, int x2, int y2)
@@ -387,7 +387,7 @@ struct wxIDirectFBWindow : public wxDfbWrapper<IDirectFBWindow>
     bool Resize(int w, int h)
         { return Check(m_ptr->Resize(m_ptr, w, h)); }
 
-    bool SetOpacity(__u8 opacity)
+    bool SetOpacity(u8 opacity)
         { return Check(m_ptr->SetOpacity(m_ptr, opacity)); }
 
     bool SetStackingClass(DFBWindowStackingClass klass)
