@@ -4,7 +4,7 @@
 // Author:      Michael Bedward (based on code by Julian Smart, Robin Dunn)
 // Modified by: Santiago Palacios
 // Created:     1/08/1999
-// RCS-ID:      $Id: grid.h 50201 2007-11-23 21:21:25Z VS $
+// RCS-ID:      $Id: grid.h 53135 2008-04-12 02:31:04Z VZ $
 // Copyright:   (c) Michael Bedward
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -2006,7 +2006,7 @@ private:
     // Calculate the minimum acceptable size for labels area
     wxCoord CalcColOrRowLabelAreaMinSize(bool column /* or row? */);
 
-    friend class WXDLLIMPEXP_ADV wxGridSelection;
+    friend class WXDLLIMPEXP_FWD_ADV wxGridSelection;
 
     DECLARE_DYNAMIC_CLASS( wxGrid )
     DECLARE_EVENT_TABLE()
