@@ -4,7 +4,7 @@
 // Author:      Vaclav Slavik
 //              based on MGL implementation
 // Created:     2006-08-16
-// RCS-ID:      $Id: app.cpp 43999 2006-12-18 17:24:06Z VS $
+// RCS-ID:      $Id: app.cpp 53607 2008-05-16 15:21:40Z SN $
 // Copyright:   (c) 2006 REA Elektronik GmbH
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -137,8 +137,9 @@ bool wxApp::Yield(bool onlyIfNeeded)
 
     wxLog::Suspend();
 
-    if ( wxEventLoop::GetActive() )
-        wxEventLoop::GetActive()->Yield();
+    // A guarentee that there will be an active event loop:
+    wxEventLoopGuarantor dummyLoopIfNeeded;
+    wxEventLoop::GetActive()->Yield();
 
     // it's necessary to call ProcessIdle() to update the frames sizes which
     // might have been changed (it also will update other things set from
