@@ -2,7 +2,7 @@
 // Name:        src/gtk/menu.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: menu.cpp 48053 2007-08-13 17:07:01Z JS $
+// Id:          $Id: menu.cpp 54173 2008-06-12 20:25:44Z JS $
 // Copyright:   (c) 1998 Robert Roebling
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -773,12 +773,15 @@ wxMenuItem::~wxMenuItem()
 /* static */
 wxString wxMenuItemBase::GetLabelFromText(const wxString& text)
 {
+#if 0
     // The argument to this function will now always be in wxWidgets standard label
     // format, not GTK+ format, so we do what the other ports do.
+    // Actually, m_text is in GTK+ format this function is used on it
+    // we need to retain the original code below.
 
     return wxStripMenuCodes(text);
 
-#if 0
+#else
     wxString label;
 
     for ( const wxChar *pc = text.c_str(); *pc; pc++ )
@@ -905,10 +908,12 @@ void wxMenuItem::SetText( const wxString& str )
     {
         // if the accelerator was taken from a stock ID, just get it back from GTK+ stock
         if (wxGetStockGtkAccelerator(stockid, &accel_mods, &accel_key))
-            gtk_widget_remove_accelerator( GTK_WIDGET(m_menuItem),
-                                           m_parentMenu->m_accel,
-                                           accel_key,
-                                           accel_mods );
+            gtk_widget_add_accelerator( GTK_WIDGET(m_menuItem),
+                                        "activate",
+                                        m_parentMenu->m_accel,
+                                        accel_key,
+                                        accel_mods,
+                                        GTK_ACCEL_VISIBLE);
     }
 }
 
@@ -1043,7 +1048,8 @@ void wxMenu::Init()
     m_menu = gtk_menu_new();
     // NB: keep reference to the menu so that it is not destroyed behind
     //     our back by GTK+ e.g. when it is removed from menubar:
-    gtk_widget_ref(m_menu);
+    g_object_ref(m_menu);
+    gtk_object_sink(GTK_OBJECT(m_menu));
 
     m_owner = (GtkWidget*) NULL;
 
@@ -1069,12 +1075,10 @@ void wxMenu::Init()
 
 wxMenu::~wxMenu()
 {
-   WX_CLEAR_LIST(wxMenuItemList, m_items);
-
    if ( GTK_IS_WIDGET( m_menu ))
    {
        // see wxMenu::Init
-       gtk_widget_unref( m_menu );
+       g_object_unref(m_menu);
        g_object_unref( m_accel );
        // if the menu is inserted in another menu at this time, there was
        // one more reference to it:
@@ -1312,9 +1316,10 @@ wxMenuItem *wxMenu::DoRemove(wxMenuItem *item)
     if ( !wxMenuBase::DoRemove(item) )
         return (wxMenuItem *)NULL;
 
-    // TODO: this code doesn't delete the item factory item and this seems
-    //       impossible as of GTK 1.2.6.
-    gtk_widget_destroy( item->GetMenuItem() );
+    GtkWidget* mitem = item->GetMenuItem();
+    gtk_menu_item_set_submenu(GTK_MENU_ITEM(mitem), NULL);
+    gtk_widget_destroy(mitem);
+    item->SetMenuItem(NULL);
 
     return item;
 }
