@@ -2,7 +2,7 @@
 // Name:        src/gtk/notebook.cpp
 // Purpose:
 // Author:      Robert Roebling
-// Id:          $Id: notebook.cpp 43051 2006-11-04 18:29:28Z RR $
+// Id:          $Id: notebook.cpp 52841 2008-03-27 00:28:45Z VZ $
 // Copyright:   (c) 1998 Robert Roebling, Vadim Zeitlin
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -60,13 +60,13 @@ public:
     wxGtkNotebookPage()
     {
         m_image = -1;
-        m_page = (GtkNotebookPage *) NULL;
-        m_box = (GtkWidget *) NULL;
+        m_page = NULL;
+        m_box = NULL;
     }
 
     wxString           m_text;
     int                m_image;
-    GtkNotebookPage   *m_page;
+    GtkWidget         *m_page;
     GtkLabel          *m_label;
     GtkWidget         *m_box;     // in which the label and image are packed
 };
@@ -122,9 +122,10 @@ static void gtk_page_size_callback( GtkWidget *WXUNUSED(widget), GtkAllocation*
     if (g_isIdle)
         wxapp_install_idle_handler();
 
-    if ((win->m_x == alloc->x) &&
-        (win->m_y == alloc->y) &&
-        (win->m_width == alloc->width) &&
+    // wxWindowGTK::SetSize() ignores position if parent does not have
+    // m_wxwindow, and infinite sizing loop can result if position is part
+    // of this check. See patch 1907189.
+    if ((win->m_width == alloc->width) &&
         (win->m_height == alloc->height))
     {
         return;
@@ -258,9 +259,9 @@ static void wxInsertChildInNotebook( wxNotebook* parent, wxWindow* child )
 // wxNotebook
 //-----------------------------------------------------------------------------
 
-IMPLEMENT_DYNAMIC_CLASS(wxNotebook,wxControl)
+IMPLEMENT_DYNAMIC_CLASS(wxNotebook,wxBookCtrlBase)
 
-BEGIN_EVENT_TABLE(wxNotebook, wxControl)
+BEGIN_EVENT_TABLE(wxNotebook, wxBookCtrlBase)
     EVT_NAVIGATION_KEY(wxNotebook::OnNavigationKey)
 END_EVENT_TABLE()
 
@@ -402,7 +403,7 @@ int wxNotebook::DoSetSelection( size_t page, int flags )
     if ( !(flags & SetSelection_SendEvent) )
     {
         // reconnect to signals
-        
+
         g_signal_connect (m_widget, "switch_page",
                           G_CALLBACK (gtk_notebook_page_changing_callback), this);
 
@@ -563,7 +564,12 @@ bool wxNotebook::DeleteAllPages()
 
 wxNotebookPage *wxNotebook::DoRemovePage( size_t page )
 {
-    wxNotebookPage *client = wxNotebookBase::DoRemovePage(page);
+    // We cannot remove the page yet, as GTK sends the "switch_page"
+    // signal before it has removed the notebook-page from its
+    // corresponding list. Thus, if we were to remove the page from
+    // m_pages at this point, the two lists of pages would be out
+    // of sync during the PAGE_CHANGING/PAGE_CHANGED events.
+    wxNotebookPage *client = GetPage(page);
     if ( !client )
         return NULL;
 
@@ -572,13 +578,16 @@ wxNotebookPage *wxNotebook::DoRemovePage( size_t page )
 
     // we don't need to unparent the client->m_widget; GTK+ will do
     // that for us (and will throw a warning if we do it!)
-
     gtk_notebook_remove_page( GTK_NOTEBOOK(m_widget), page );
 
+    // It's safe to remove the page now.
+    wxASSERT_MSG(GetPage(page) == client, wxT("pages changed during delete"));
+    wxNotebookBase::DoRemovePage(page);
+
     wxGtkNotebookPage* p = GetNotebookPage(page);
     m_pagesData.DeleteObject(p);
     delete p;
-
+    
     return client;
 }
 
@@ -617,18 +626,23 @@ bool wxNotebook::InsertPage( size_t position,
 
     m_pages.Insert(win, position);
 
+    // set the label image and text
+    // this must be done before adding the page, as GetPageText
+    // and GetPageImage will otherwise return wrong values in
+    // the page-changed event that results from inserting the
+    // first page.
+    nb_page->m_image = imageId;
+    nb_page->m_text = wxStripMenuCodes(text);
+
     nb_page->m_box = gtk_hbox_new( FALSE, 1 );
     gtk_container_set_border_width((GtkContainer*)nb_page->m_box, 2);
 
     g_signal_connect (win->m_widget, "size_allocate",
                       G_CALLBACK (gtk_page_size_callback), win);
 
-    gtk_notebook_insert_page( notebook, win->m_widget, nb_page->m_box, position );
+    gtk_notebook_insert_page(notebook, win->m_widget, nb_page->m_box, position);
 
-    nb_page->m_page = (GtkNotebookPage*) g_list_last(notebook->children)->data;
-
-    /* set the label image */
-    nb_page->m_image = imageId;
+    nb_page->m_page = gtk_notebook_get_nth_page(notebook, position);
 
     if (imageId != -1)
     {
@@ -641,10 +655,6 @@ bool wxNotebook::InsertPage( size_t position,
     }
 
     /* set the label text */
-
-    nb_page->m_text = wxStripMenuCodes(text);
-    if (nb_page->m_text.empty()) nb_page->m_text = wxEmptyString;
-
     nb_page->m_label = GTK_LABEL( gtk_label_new(wxGTK_CONV(nb_page->m_text)) );
     gtk_box_pack_end( GTK_BOX(nb_page->m_box), GTK_WIDGET(nb_page->m_label), FALSE, FALSE, m_padding );
 
@@ -658,7 +668,7 @@ bool wxNotebook::InsertPage( size_t position,
 
     /* show the label */
     gtk_widget_show( GTK_WIDGET(nb_page->m_label) );
-    
+
     if (select && (m_pagesData.GetCount() > 1))
     {
         SetSelection( position );
