@@ -4,7 +4,7 @@
 // Author:      Jethro Grassie / Kevin Ollivier
 // Modified by:
 // Created:     2004-4-16
-// RCS-ID:      $Id: webkit.mm 43831 2006-12-06 04:37:32Z KO $
+// RCS-ID:      $Id: webkit.mm 53798 2008-05-28 06:12:34Z RD $
 // Copyright:   (c) Jethro Grassie / Kevin Ollivier
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -309,6 +309,20 @@ wxWebKitBeforeLoadEvent::wxWebKitBeforeLoadEvent( wxWindow* win )
     SetId(win->GetId());
 }
 
+
+IMPLEMENT_DYNAMIC_CLASS( wxWebKitNewWindowEvent, wxCommandEvent )
+
+DEFINE_EVENT_TYPE( wxEVT_WEBKIT_NEW_WINDOW )
+
+wxWebKitNewWindowEvent::wxWebKitNewWindowEvent( wxWindow* win )
+{
+    SetEventType( wxEVT_WEBKIT_NEW_WINDOW);
+    SetEventObject( win );
+    SetId(win->GetId());
+}
+
+
+
 //---------------------------------------------------------
 // helper functions for NSString<->wxString conversion
 //---------------------------------------------------------
@@ -845,12 +859,25 @@ void wxWebKitCtrl::MacVisibilityChanged(){
     if (webKitWindow && webKitWindow->GetEventHandler())
         webKitWindow->GetEventHandler()->ProcessEvent(thisEvent);
 
-	if (thisEvent.IsCancelled())
+    if (thisEvent.IsCancelled())
         [listener ignore];
     else
         [listener use];
 }
 
+- (void)webView:(WebView *)sender decidePolicyForNewWindowAction:(NSDictionary *)actionInformation request:(NSURLRequest *)request newFrameName:(NSString *)frameName decisionListener:(id < WebPolicyDecisionListener >)listener
+{
+    wxWebKitNewWindowEvent thisEvent(webKitWindow);
+
+    NSString *url = [[request URL] absoluteString];
+    thisEvent.SetURL( wxStringWithNSString( url ) );
+    thisEvent.SetTargetName( wxStringWithNSString( frameName ) );
+    
+    if (webKitWindow && webKitWindow->GetEventHandler())
+        webKitWindow->GetEventHandler()->ProcessEvent(thisEvent);
+
+    [listener use];
+}
 @end
 
 #endif //wxUSE_WEBKIT
