@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     01/02/97
-// RCS-ID:      $Id: dcprint.cpp 43106 2006-11-06 07:49:14Z SC $
+// RCS-ID:      $Id: dcprint.cpp 53928 2008-06-02 18:22:15Z RD $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -92,10 +92,12 @@ wxMacCarbonPrinterDC::wxMacCarbonPrinterDC( wxPrintData* data )
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_5 
     PMPrinter printer;
     PMSessionGetCurrentPrinter(native->m_macPrintSession, &printer);
-    PMPrinterGetOutputResolution( printer, native->m_macPrintSettings, &res) ;
-#else
-    m_err = PMGetResolution((PMPageFormat) (native->m_macPageFormat), &res);
+    m_err = PMPrinterGetOutputResolution( printer, native->m_macPrintSettings, &res) ;
+    if ( m_err )
+        // fallback to the old API if there was an error
 #endif
+        m_err = PMGetResolution((PMPageFormat) (native->m_macPageFormat), &res);
+    
     m_ppi = wxSize(int(res.hRes), int(res.vRes));
 }
 
@@ -153,10 +155,12 @@ bool wxMacCarbonPrinterDC::StartDoc(  wxPrinterDC* dc , const wxString& WXUNUSED
 #if MAC_OS_X_VERSION_MIN_REQUIRED >= MAC_OS_X_VERSION_10_5 
     PMPrinter printer;
     PMSessionGetCurrentPrinter(native->m_macPrintSession, &printer);
-    PMPrinterGetOutputResolution( printer, native->m_macPrintSettings, &res) ;
-#else
-    m_err = PMGetResolution((PMPageFormat) (native->m_macPageFormat), &res);
+    m_err = PMPrinterGetOutputResolution( printer, native->m_macPrintSettings, &res) ;
+    if ( m_err )
+        // fallback to the old API if there was an error
 #endif
+        m_err = PMGetResolution((PMPageFormat) (native->m_macPageFormat), &res);
+    
     m_ppi = wxSize(int(res.hRes), int(res.vRes));
     return true ;
 }
