@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by: Vadim Zeitlin (owner drawn stuff)
 // Created:
-// RCS-ID:      $Id: listbox.cpp,v 1.136 2006/10/31 08:49:41 RD Exp $
+// RCS-ID:      $Id: listbox.cpp 45959 2007-05-11 13:05:02Z VZ $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 ///////////////////////////////////////////////////////////////////////////////
@@ -798,7 +798,7 @@ bool wxListBox::MSWOnDraw(WXDRAWITEMSTRUCT *item)
     if ( itemID == (UINT)-1 )
         return false;
 
-    long data = ListBox_GetItemData(GetHwnd(), pStruct->itemID);
+    LRESULT data = ListBox_GetItemData(GetHwnd(), pStruct->itemID);
 
     wxCHECK( data && (data != LB_ERR), false );
 
