@@ -4,7 +4,7 @@
 // Author:      Julian Smart
 // Modified by:
 // Created:     2005-09-30
-// RCS-ID:      $Id: richtexthtml.cpp 48598 2007-09-07 14:15:46Z JS $
+// RCS-ID:      $Id: richtexthtml.cpp 51375 2008-01-25 16:38:18Z JS $
 // Copyright:   (c) Julian Smart
 // Licence:     wxWindows licence
 /////////////////////////////////////////////////////////////////////////////
@@ -85,8 +85,6 @@ bool wxRichTextHTMLHandler::DoSaveFile(wxRichTextBuffer *buffer, wxOutputStream&
     if ((GetFlags() & wxRICHTEXT_HANDLER_NO_HEADER_FOOTER) == 0)
         str << wxT("<html><head></head><body>\n");
 
-    str << wxT("<table border=0 cellpadding=0 cellspacing=0><tr><td width=\"100%\">");
-
     OutputFont(currentParaStyle, str);
 
     m_font = false;
@@ -131,7 +129,7 @@ bool wxRichTextHTMLHandler::DoSaveFile(wxRichTextBuffer *buffer, wxOutputStream&
                 }
 
                 wxRichTextImage* image = wxDynamicCast(obj, wxRichTextImage);
-                if( image && !image->IsEmpty())
+                if( image && (!image->IsEmpty() || image->GetImageBlock().GetData()))
                     WriteImage( image, stream );
 
                 node2 = node2->GetNext();
@@ -148,8 +146,6 @@ bool wxRichTextHTMLHandler::DoSaveFile(wxRichTextBuffer *buffer, wxOutputStream&
 
     str << wxT("</font>");
 
-    str << wxT("</td></tr></table><p>");
-
     if ((GetFlags() & wxRICHTEXT_HANDLER_NO_HEADER_FOOTER) == 0)
         str << wxT("</body></html>");
 
@@ -219,9 +215,7 @@ void wxRichTextHTMLHandler::BeginParagraphFormatting(const wxTextAttrEx& WXUNUSE
 {
     if (thisStyle.HasPageBreak())
     {
-        str << wxT("</tr></td></table>");
         str << wxT("<div style=\"page-break-after:always\"></div>\n");
-        str << wxT("<table border=0 cellpadding=0 cellspacing=0><tr><td width=\"100%\">");
     }
 
     if (thisStyle.HasLeftIndent() && thisStyle.GetLeftIndent() != 0)
@@ -293,9 +287,11 @@ void wxRichTextHTMLHandler::EndParagraphFormatting(const wxTextAttrEx& WXUNUSED(
         if (thisStyle.HasFont())
             stream << wxT("</font>");
 
-        stream << wxT("</td></tr></table>\n");
+        stream << wxT("</td></tr></table></p>\n");
         m_inTable = false;
     }
+    else
+        stream << wxT("</p>\n");
 }
 
 /// Closes lists to level (-1 means close all)
