@@ -17,36 +17,6 @@ ID_SelectionPen = wx.NewId()
 ID_SeparatorColour = wx.NewId()
 
 
-#----------------------------------------------------------------------
-# Some icons for the demo
-#----------------------------------------------------------------------
-def GetMondrianData():
-    return \
-'\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00 \x00\x00\x00 \x08\x06\x00\
-\x00\x00szz\xf4\x00\x00\x00\x04sBIT\x08\x08\x08\x08|\x08d\x88\x00\x00\x00qID\
-ATX\x85\xed\xd6;\n\x800\x10E\xd1{\xc5\x8d\xb9r\x97\x16\x0b\xad$\x8a\x82:\x16\
-o\xda\x84pB2\x1f\x81Fa\x8c\x9c\x08\x04Z{\xcf\xa72\xbcv\xfa\xc5\x08 \x80r\x80\
-\xfc\xa2\x0e\x1c\xe4\xba\xfaX\x1d\xd0\xde]S\x07\x02\xd8>\xe1wa-`\x9fQ\xe9\
-\x86\x01\x04\x10\x00\\(Dk\x1b-\x04\xdc\x1d\x07\x14\x98;\x0bS\x7f\x7f\xf9\x13\
-\x04\x10@\xf9X\xbe\x00\xc9 \x14K\xc1<={\x00\x00\x00\x00IEND\xaeB`\x82' 
-
-
-def GetMondrianBitmap():
-    return wx.BitmapFromImage(GetMondrianImage())
-
-
-def GetMondrianImage():
-    import cStringIO
-    stream = cStringIO.StringIO(GetMondrianData())
-    return wx.ImageFromStream(stream)
-
-
-def GetMondrianIcon():
-    icon = wx.EmptyIcon()
-    icon.CopyFromBitmap(GetMondrianBitmap())
-    return icon
-
-
 #----------------------------------------------------------------------
 
 class SettingsPanel(wx.MiniFrame):
@@ -474,7 +444,7 @@ class ButtonPanelDemo(wx.Frame):
         self.useredited = False
         self.hassettingpanel = False
 
-        self.SetIcon(GetMondrianIcon())
+        self.SetIcon(images.Mondrian.GetIcon())
         self.CreateMenuBar()
 
         self.statusbar = self.CreateStatusBar(2, wx.ST_SIZEGRIP)
@@ -501,10 +471,10 @@ class ButtonPanelDemo(wx.Frame):
                                        style=self.style, alignment=self.alignment)
 
         self.created = False
-        self.pngs = [ (images.get_bp_btn1Bitmap(), 'label1'),
-                      (images.get_bp_btn2Bitmap(), 'label2'),
-                      (images.get_bp_btn3Bitmap(), 'label3'),
-                      (images.get_bp_btn4Bitmap(), 'label4'),
+        self.pngs = [ (images._bp_btn1.GetBitmap(), 'label1'),
+                      (images._bp_btn2.GetBitmap(), 'label2'),
+                      (images._bp_btn3.GetBitmap(), 'label3'),
+                      (images._bp_btn4.GetBitmap(), 'label4'),
                       ]
         self.CreateButtons()
         self.SetProperties()
