@@ -3,8 +3,8 @@
 # Name: browser.py                                                            #
 # Purpose: UI portion of the FileBrowser Plugin                               #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
@@ -14,25 +14,55 @@ FileBrowser Plugin.
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: browser.py 49511 2007-10-29 07:58:06Z CJP $"
-__revision__ = "$Revision: 49511 $"
+__svnid__ = "$Id: browser.py 53543 2008-05-10 18:32:25Z CJP $"
+__revision__ = "$Revision: 53543 $"
 
 #-----------------------------------------------------------------------------#
 # Imports
 import os
+import sys
 import stat
+import zipfile
+import shutil
+import subprocess
+import threading
 import wx
+
+# Editra Library Modules
 import ed_glob
+import ed_msg
 import ed_menu
 import syntax.syntax
 import util
 from profiler import Profile_Get, Profile_Set
+from eclib import platebtn
+
+# Local Imports
+import FileInfo
+import Trash
 
 #-----------------------------------------------------------------------------#
 # Globals
 PANE_NAME = u'FileBrowser'
 ID_BROWSERPANE = wx.NewId()
 ID_FILEBROWSE = wx.NewId()
+
+# Configure Platform specific commands
+FILEMAN_CMD = util.GetFileManagerCmd()
+if wx.Platform == '__WXMAC__':
+    FILEMAN = 'Finder'
+elif wx.Platform == '__WXMSW__':
+    FILEMAN = 'Explorer'
+else: # Other/Linux
+    FILEMAN = FILEMAN_CMD.title()
+
+def TrashString():
+    """Get the trash description string"""
+    if wx.Platform == '__WXMSW__':
+        return _("Move to Recycle Bin")
+    else:
+        return _("Move to Trash")
+
 _ = wx.GetTranslation
 #-----------------------------------------------------------------------------#
 
@@ -40,19 +70,12 @@ class BrowserMenuBar(wx.Panel):
     """Creates a menubar with """
     ID_MARK_PATH = wx.NewId()
     ID_OPEN_MARK = wx.NewId()
-    ID_PATHS = wx.NewId()
     ID_REMOVE_MARK = wx.NewId()
 
     def __init__(self, parent):
         wx.Panel.__init__(self, parent, style=wx.NO_BORDER)
 
         # Attributes
-        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_ADD_BM), wx.ART_MENU)
-        self._menub = wx.BitmapButton(self, self.ID_PATHS, 
-                                      bmp, style=wx.NO_BORDER)
-        tt = wx.ToolTip(_("Pathmarks"))
-        self._menub.SetToolTip(tt)
-        self._menu = ed_menu.EdMenu()
         self._saved = ed_menu.EdMenu()
         self._rmpath = ed_menu.EdMenu()
         self._ids = list()  # List of ids of menu items
@@ -68,26 +91,42 @@ class BrowserMenuBar(wx.Panel):
         self.SetToolTip(tt)
 
         # Build Menus
-        self._menu.Append(self.ID_MARK_PATH, _("Save Selected Paths"))
-        self._menu.AppendMenu(self.ID_OPEN_MARK, 
-                              _("Jump to Saved Path"), self._saved)
-        self._menu.AppendSeparator()
-        self._menu.AppendMenu(self.ID_REMOVE_MARK, 
-                              _("Remove Saved Path"), self._rmpath)
+        menu = ed_menu.EdMenu()
+        menu.Append(self.ID_MARK_PATH, _("Save Selected Paths"))
+        menu.AppendMenu(self.ID_OPEN_MARK, 
+                        _("Jump to Saved Path"), self._saved)
+        menu.AppendSeparator()
+        menu.AppendMenu(self.ID_REMOVE_MARK, 
+                        _("Remove Saved Path"), self._rmpath)
+
+        # Button
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_ADD_BM), wx.ART_MENU)
+        self.menub = platebtn.PlateButton(self, bmp=bmp,
+                                          style=platebtn.PB_STYLE_NOBG)
+        self.menub.SetToolTipString(_("Pathmarks"))
+        self.menub.SetMenu(menu)
+
 
         # Layout bar
-        self._sizer = wx.BoxSizer(wx.VERTICAL)
-        self._sizer.Add((3, 3))
+        sizer = wx.BoxSizer(wx.VERTICAL)
+        sizer.Add((1, 1))
         men_sz = wx.BoxSizer(wx.HORIZONTAL)
         men_sz.Add((6, 6))
-        men_sz.Add(self._menub, 0, wx.ALIGN_LEFT)
-        self._sizer.Add(men_sz)
-        self._sizer.Add((3, 3))
-        self.SetSizer(self._sizer)
+        men_sz.Add(self.menub, 0, wx.ALIGN_LEFT)
+        sizer.Add(men_sz)
+        sizer.Add((1, 1))
+        self.SetSizer(sizer)
 
         # Event Handlers
-        self._menub.Bind(wx.EVT_BUTTON, self.OnButton, id=self.ID_PATHS)
-        self.Bind(wx.EVT_PAINT, self.OnPaint)
+        ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
+        self.Bind(wx.EVT_BUTTON, lambda evt: self.menub.ShowMenu(), self.menub)
+        # Due to transparency issues dont do painting on gtk
+        if wx.Platform != '__WXGTK__':
+            self.Bind(wx.EVT_PAINT, self.OnPaint)
+
+    def __del__(self):
+        """Unsubscribe from messages"""
+        ed_msg.Unsubscribe(self.OnThemeChanged)
 
     # XXX maybe change to list the more recently added items near the top
     def AddItem(self, label):
@@ -129,35 +168,34 @@ class BrowserMenuBar(wx.Panel):
         """Returns the menu containg the saved items"""
         return self._saved
 
-    def OnButton(self, evt):
-        """Pops the menu open when the button has been clicked on"""
-        e_id = evt.GetId()
-        if e_id == self.ID_PATHS:
-            men_rect = self._menub.GetRect()
-            pos = wx.Point(0, men_rect.GetY() + men_rect.GetHeight())
-            self._menub.PopupMenu(self._menu, pos)
-        else:
-            evt.Skip()
-
     def OnPaint(self, evt):
         """Paints the background of the menubar"""
         dc = wx.PaintDC(self)
         gc = wx.GraphicsContext.Create(dc)
-        col1 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), -50)
-        col2 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), 50)
-        grad = gc.CreateLinearGradientBrush(0, 1, 0, 29, col2, col1)
+        col1 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), -15)
+        col2 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), 40)
         rect = self.GetRect()
+        grad = gc.CreateLinearGradientBrush(0, 1, 0, rect.height, col2, col1)
 
         # Create the background path
         path = gc.CreatePath()
         path.AddRectangle(0, 0, rect.width - 0.5, rect.height - 0.5)
 
-        gc.SetPen(wx.Pen(util.AdjustColour(col1, -60), 1))
+        gc.SetPen(wx.Pen(util.AdjustColour(col1, -20), 1))
         gc.SetBrush(grad)
         gc.DrawPath(path)
 
         evt.Skip()
 
+    def OnThemeChanged(self, msg):
+        """Update the buttons icon when the icon theme changes
+        @param msg: Message Object
+
+        """
+        bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_ADD_BM), wx.ART_MENU)
+        self.menub.SetBitmap(bmp)
+        self.menub.Refresh()
+
     def RemoveItemById(self, path_id):
         """Removes a given menu item from both the saved
         and removed lists using the id as a lookup.
@@ -191,7 +229,9 @@ class BrowserPane(wx.Panel):
         self._browser = FileBrowser(self, ID_FILEBROWSE, 
                                     dir=wx.GetHomeDir(), 
                                     size=(200, -1),
-                                    style=wx.DIRCTRL_SHOW_FILTERS | wx.BORDER_SUNKEN,
+                                    style=wx.DIRCTRL_SHOW_FILTERS | 
+                                          wx.DIRCTRL_EDIT_LABELS | 
+                                          wx.BORDER_SUNKEN,
                                     filter=filters)
         self._config = PathMarkConfig(ed_glob.CONFIG['CACHE_DIR'])
         for item in self._config.GetItemLabels():
@@ -199,6 +239,8 @@ class BrowserPane(wx.Panel):
         self._showh_cb = wx.CheckBox(self, self.ID_SHOW_HIDDEN, 
                                      _("Show Hidden Files"))
         self._showh_cb.SetValue(False)
+        if wx.Platform == '__WXMAC__':
+            self._showh_cb.SetWindowVariant(wx.WINDOW_VARIANT_SMALL)
 
         #---- Add Menu Items ----#
         viewm = self._mw.GetMenuBar().GetMenuByName("view")
@@ -222,11 +264,14 @@ class BrowserPane(wx.Panel):
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
         self.Bind(wx.EVT_MENU, self.OnMenu)
         self.Bind(wx.EVT_PAINT, self.OnPaint)
-        viewm.Bind(wx.EVT_MENU_OPEN, self.UpdateMenuItem)
+
+        # Messages
+        ed_msg.Subscribe(self.OnUpdateFont, ed_msg.EDMSG_DSP_FONT)
 
     def __del__(self):
         """Save the config before we get destroyed"""
         self._config.Save()
+        ed_msg.Unsubscribe(self.OnUpdateFont)
 
     def OnCheck(self, evt):
         """Toggles visibility of hidden files on and off"""
@@ -241,8 +286,6 @@ class BrowserPane(wx.Panel):
         else:
             evt.Skip()
 
-    # TODO Add input method so that paths can be given custom
-    #      labels
     # TODO after a jump the window should be properly rescrolled
     #      to have the jumped-to path at the top when possible
     def OnMenu(self, evt):
@@ -276,8 +319,8 @@ class BrowserPane(wx.Panel):
         """Paints the background of the panel"""
         dc = wx.PaintDC(self)
         gc = wx.GraphicsContext.Create(dc)
-        col1 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), -50)
-        col2 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), 50)
+        col1 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), -15)
+        col2 = util.AdjustColour(wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE), 40)
         rect = self.GetRect()
         x = 0
         y = rect.height - (self._showh_cb.GetSize()[1] + 6)
@@ -290,7 +333,7 @@ class BrowserPane(wx.Panel):
         path.AddRectangle(x, y, rect.width - 0.5, 
                           self._showh_cb.GetSize()[1] + 6)
 
-        gc.SetPen(wx.Pen(util.AdjustColour(col1, -60), 1))
+        gc.SetPen(wx.Pen(util.AdjustColour(col1, -20), 1))
         gc.SetBrush(grad)
         gc.DrawPath(path)
 
@@ -303,101 +346,178 @@ class BrowserPane(wx.Panel):
             pane = mgr.GetPane(PANE_NAME)
             if pane.IsShown():
                 pane.Hide()
-                Profile_Set('SHOW_FB', False)
             else:
                 pane.Show()
-                Profile_Set('SHOW_FB', True)
             mgr.Update()
         else:
             evt.Skip()
 
-    def UpdateMenuItem(self, evt):
-        """Update the check mark for the menu item"""
-        mgr = self._mw.GetFrameManager()
-        pane = mgr.GetPane(PANE_NAME)
-        if pane.IsShown():
-            self._mi.Check(True)
-        else:
-            self._mi.Check(False)
-        evt.Skip()
+    def OnUpdateFont(self, msg):
+        """Update the ui font when a message comes saying to do so."""
+        font = msg.GetData()
+        if isinstance(font, wx.Font) and not font.IsNull():
+            for child in (self, self._browser, self._browser.GetTreeCtrl()):
+                if hasattr(child, 'SetFont'):
+                    child.SetFont(font)
+
+    def OnUpdateMenu(self, evt):
+        """UpdateUI handler for the panels menu item, to update the check
+        mark.
+        @param evt: wx.UpdateUIEvent
+
+        """
+        pane = self._mw.GetFrameManager().GetPane(PANE_NAME)
+        evt.Check(pane.IsShown())
 
 #-----------------------------------------------------------------------------#
+# Menu Id's
+ID_EDIT = wx.NewId()
+ID_OPEN = wx.NewId()
+ID_REVEAL = wx.NewId()
+ID_GETINFO = wx.NewId()
+ID_RENAME = wx.NewId()
+ID_NEW_FOLDER = wx.NewId()
+ID_NEW_FILE = wx.NewId()
+ID_DELETE = wx.NewId()
+ID_DUPLICATE = wx.NewId()
+ID_ARCHIVE = wx.NewId()
 
 class FileBrowser(wx.GenericDirCtrl):
-    """A hack job done to make the genericdirctrl more useful
-    and work with Editra's art provider.
-    @todo: write my own dirctrl
-
-    """
+    """Custom directory control for displaying and managing files"""
     def __init__(self, parent, id, dir=u'', pos=wx.DefaultPosition,
-                 size=wx.DefaultSize, style=wx.DIRCTRL_SHOW_FILTERS,
+                 size=wx.DefaultSize,
+                 style=wx.DIRCTRL_SHOW_FILTERS|wx.DIRCTRL_EDIT_LABELS,
                  filter=wx.EmptyString, defaultFilter=0):
         wx.GenericDirCtrl.__init__(self, parent, id, dir, pos, 
                                    size, style, filter, defaultFilter)
 
         # Attributes
         self._tree = self.GetTreeCtrl()
+        self._treeId = 0                # id of TreeItem that was last rclicked
+        self._fmenu = self._MakeMenu()
         
         # Set custom styles
         self._tree.SetWindowStyle(self._tree.GetWindowStyle() | wx.TR_MULTIPLE)
         self._tree.Refresh()
-
-        # HACK if the GenericDirCtrl ever changes the order of the images used 
-        #      in it this will have to be updated accordingly
-        if Profile_Get('ICONS', 'str', 'default').lower() != u'default':
-            bmp1 = wx.ArtProvider.GetBitmap(str(ed_glob.ID_FOLDER), wx.ART_MENU)
-            fbmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_FILE), wx.ART_MENU)
-            self._imglst = self._tree.GetImageList()
-            ids = [ed_glob.ID_FOLDER, ed_glob.ID_OPEN, ed_glob.ID_COMPUTER, 
-                   ed_glob.ID_HARDDISK, ed_glob.ID_CDROM, ed_glob.ID_FLOPPY, 
-                   ed_glob.ID_USB, ed_glob.ID_FILE, ed_glob.ID_BIN_FILE]
-
-            for art in ids:
-                bmp = wx.ArtProvider.GetBitmap(str(art), wx.ART_MENU)
-                if bmp.IsOk():
-                    self._imglst.Replace(ids.index(art), bmp)
+        self._imglst = self._tree.GetImageList()
+        self._SetupIcons()
 
         # Event Handlers
         self.Bind(wx.EVT_TREE_ITEM_ACTIVATED, self.OnOpen)
-        if wx.Platform == '__WXMSW__':
-            self._tree.Bind(wx.EVT_KEY_UP, self.OnOpen)
+        self.Bind(wx.EVT_TREE_ITEM_RIGHT_CLICK, self.OnContext)
+        self.Bind(wx.EVT_MENU, self.OnMenu)
+        ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
+        ed_msg.Subscribe(self.OnPageChange, ed_msg.EDMSG_UI_NB_CHANGED)
 #         self.Bind(wx.EVT_TREE_BEGIN_DRAG, self.OnDragStart)
 #         self.Bind(wx.EVT_TREE_END_DRAG, self.OnDragEnd)
 
+    def __del__(self):
+        """Unsubscribe from messages"""
+        ed_msg.Unsubscribe(self.OnThemeChanged)
+        ed_msg.Unsubscribe(self.OnPageChange)
+
+    def _SetupIcons(self):
+        """If a custom theme is in use by Editra set the icons of the
+        directory control to match.
+
+        """
+        # HACK if the GenericDirCtrl ever changes the order of the images used 
+        #      in it this will have to be updated accordingly
+        ids = [ed_glob.ID_FOLDER, ed_glob.ID_OPEN, ed_glob.ID_COMPUTER, 
+               ed_glob.ID_HARDDISK, ed_glob.ID_CDROM, ed_glob.ID_FLOPPY, 
+               ed_glob.ID_USB, ed_glob.ID_FILE, ed_glob.ID_BIN_FILE]
+
+        for idx, art in enumerate(ids):
+            bmp = wx.ArtProvider.GetBitmap(str(art), wx.ART_MENU)
+            if bmp.IsOk():
+                self._imglst.Replace(idx, bmp)
+
+    def _MakeMenu(self):
+        """Setup the context menu"""
+        menu = wx.Menu()
+
+        # MenuItems
+        items = [(ID_EDIT, _("Edit"), None),
+                 (ID_OPEN, _("Open with " + FILEMAN), ed_glob.ID_OPEN),
+                 (ID_REVEAL, _("Reveal in " + FILEMAN), None),
+                 (wx.ID_SEPARATOR, '', None),
+                 (ID_GETINFO, _("Get Info"), None),
+                 (ID_RENAME, _("Rename"), None),
+                 (wx.ID_SEPARATOR, '', None),
+                 (ID_NEW_FOLDER, _("New Folder"), ed_glob.ID_FOLDER),
+                 (ID_NEW_FILE, _("New File"), ed_glob.ID_NEW),
+                 (wx.ID_SEPARATOR, '', None),
+                 (ID_DUPLICATE, _("Duplicate"), None),
+                 (ID_ARCHIVE, _("Create Archive of \"%s\"") % '', None),
+                 (wx.ID_SEPARATOR, '', None),
+                 (ID_DELETE, TrashString(), ed_glob.ID_DELETE),
+                ]
+
+        for item in items:
+            mitem = wx.MenuItem(menu, item[0], item[1])
+            if item[2] is not None:
+                bmp = wx.ArtProvider.GetBitmap(str(item[2]), wx.ART_MENU)
+                mitem.SetBitmap(bmp)
+
+            menu.AppendItem(mitem)
+        return menu
+
+    def _SCommand(self, cmd, *args):
+        """Run a tree command that requires de-hacking the windows style
+        before running. (Some commands raise errors on msw if the style is
+        not set back to TR_SINGLE before calling it.
+        @param cmd: self.METHOD
+        @keyword args: args to pass to command
+
+        """
+        style = self.GetTreeStyle()
+        self.SetTreeStyle(wx.TR_SINGLE)
+        self.Refresh()
+        rval = cmd(*args)
+        self.SetTreeStyle(style)
+        self.Refresh()
+        return rval
+
+    def GetItemPath(self, itemId):
+        """Get and return the path of the given item id
+        @param itemId: TreeItemId
+        @return: string
+
+        """
+        root = self._tree.GetRootItem()
+        start = itemId
+        atoms = [itemId]
+        while self._tree.GetItemParent(start) != root:
+            atoms.append(self._tree.GetItemParent(start))
+            start = atoms[-1]
+
+        atoms.reverse()
+        path = list()
+        for atom in atoms:
+            path.append(self._tree.GetItemText(atom))
+
+        if wx.Platform == '__WXMSW__':
+            r_txt = u''
+        else:
+            if wx.Platform == '__WXGTK__':
+                if path[0].lower() == 'home directory':
+                    path[0] = wx.GetHomeDir()
+                elif path[0].lower() == 'desktop':
+                    path.insert(0, wx.GetHomeDir())
+                else:
+                    pass
+
+            if wx.Platform == '__WXMAC__':
+                if path[0] != "/":
+                    path.pop(0)
+            r_txt = os.path.sep
+        return r_txt + os.sep.join(path)
+
     def GetPaths(self):
         """Gets a list of abs paths of the selected items"""
-        treeIds = self._tree.GetSelections()
-        root = self._tree.GetRootItem()
         ret_val = list()
-        for leaf in treeIds:
-            start = leaf
-            atoms = [leaf]
-            while self._tree.GetItemParent(start) != root:
-                atoms.append(self._tree.GetItemParent(start))
-                start = atoms[-1]
-
-            atoms.reverse()
-            path = list()
-            for atom in atoms:
-                path.append(self._tree.GetItemText(atom))
-
-            if wx.Platform == '__WXMSW__':
-                r_txt = u''
-            else:
-                if wx.Platform == '__WXGTK__':
-                    if path[0].lower() == 'home directory':
-                        path[0] = wx.GetHomeDir()
-                    elif path[0].lower() == 'desktop':
-                        path.insert(0, wx.GetHomeDir())
-                    else:
-                        pass
-
-                if wx.Platform == '__WXMAC__':
-                    if path[0] != "/":
-                        path.pop(0)
-                r_txt = os.path.sep
-
-            ret_val.append(r_txt + util.GetPathChar().join(path))
+        for leaf in self._tree.GetSelections():
+            ret_val.append(self.GetItemPath(leaf))
         return ret_val
 
     def GetScrollRange(self, orient=wx.VERTICAL):
@@ -408,6 +528,73 @@ class FileBrowser(wx.GenericDirCtrl):
         """Returns the trees current style"""
         return self._tree.GetWindowStyle()
 
+    def OnContext(self, evt):
+        """Show the popup context menu"""
+        self._fmenu.Destroy()
+        self._fmenu = self._MakeMenu()
+        self._treeId = evt.GetItem()
+        path = self.GetItemPath(self._treeId)
+        mitem = self._fmenu.FindItemById(ID_ARCHIVE)
+        if mitem != wx.NOT_FOUND:
+            mitem.SetText(_("Create Archive of \"%s\"") % \
+                          path.split(os.path.sep)[-1])
+
+        for item in (ID_DUPLICATE,):
+            self._fmenu.Enable(item, len(self._tree.GetSelections()) == 1)
+        self.PopupMenu(self._fmenu)
+
+    def OnMenu(self, evt):
+        """Handle the context menu events for performing
+        filesystem operations
+
+        """
+        e_id = evt.GetId()
+        path = self.GetItemPath(self._treeId)
+        paths = self.GetPaths()
+        ok = (False, '')
+        if e_id == ID_EDIT:
+            self.OpenFiles(paths)
+        elif e_id == ID_OPEN:
+            worker = OpenerThread(paths)
+            worker.start()
+        elif e_id == ID_REVEAL:
+            worker = OpenerThread([os.path.dirname(fname) for fname in paths])
+            worker.start()
+        elif e_id == ID_GETINFO:
+            last = None
+            for fname in paths:
+                info = FileInfo.FileInfoDlg(self.GetTopLevelParent(), fname)
+                if last is None:
+                    info.CenterOnParent()
+                else:
+                    lpos = last.GetPosition()
+                    info.SetPosition((lpos[0] + 14, lpos[1] + 14))
+                info.Show()
+                last = info
+        elif e_id == ID_RENAME:
+            self._tree.EditLabel(self._treeId)
+        elif e_id == ID_NEW_FOLDER:
+            ok = util.MakeNewFolder(path, _("Untitled_Folder"))
+        elif e_id == ID_NEW_FILE:
+            ok = util.MakeNewFile(path, _("Untitled_File") + ".txt")
+        elif e_id == ID_DUPLICATE:
+            for fname in paths:
+                ok = DuplicatePath(fname)
+        elif e_id == ID_ARCHIVE:
+            ok = MakeArchive(path)
+        elif e_id == ID_DELETE:
+            Trash.moveToTrash(paths)
+            ok = (True, os.path.dirname(path))
+        else:
+            evt.Skip()
+            return
+
+        if e_id in (ID_NEW_FOLDER, ID_NEW_FILE, ID_DUPLICATE, ID_ARCHIVE,
+                    ID_DELETE):
+            if ok[0]:
+                self.ReCreateTree()
+                self._SCommand(self.SetPath, ok[1])
+
     def OnOpen(self, evt):
         """Handles item activations events. (i.e double clicked or 
         enter is hit) and passes the clicked on file to be opened in 
@@ -415,14 +602,53 @@ class FileBrowser(wx.GenericDirCtrl):
 
         """
         files = self.GetPaths()
-        if wx.Platform == '__WXMSW__':
-            key = evt.GetKeyCode()
-            if len(files) == 1 and stat.S_ISDIR(os.stat(files[0])[0]):
+        item = self._SCommand(self.GetPath)
+        if os.path.isdir(item):
+            # Unlike Mac/Gtk, need to skip the event here to allow for
+            # expanding paths with double clicks. This however disables the
+            # ability to use Return as a key for expanding. It also doesn't
+            # seem to be possible to tell whether the activation event was
+            # caused by a double click or a key sequence.
+            if wx.Platform == '__WXMSW__':
                 evt.Skip()
-                if key == wx.WXK_RETURN:
-                    self.ExpandPath(files[0])
-                return
+            else:
+                self.ExpandPath(item)
+            return
+        else:
+            files = [ fname for fname in files if not os.path.isdir(fname) ]
+        self.OpenFiles(files)
 
+    def OnPageChange(self, msg):
+        """Syncronize selection with the notebook page changes
+        @param msg: MessageObject
+        @todo: check if message is from a page closing and avoid updates
+
+        """
+        
+        nbdata = msg.GetData()
+        page = nbdata[0].GetPage(nbdata[1])
+        path = page.GetFileName()
+        if path:
+            style = self.GetTreeStyle()
+            self.SetTreeStyle(wx.TR_SINGLE)
+            self.Refresh()
+            self.SetPath(path)
+            self.SetTreeStyle(style)
+            self.Refresh()
+
+    def OnThemeChanged(self, msg):
+        """Update the icons when the icon theme has changed
+        @param msg: Message Object
+
+        """
+        self._SetupIcons()
+        self._tree.Refresh()
+
+    def OpenFiles(self, files):
+        """Open the list of files in Editra for editing
+        @param files: list of file names
+
+        """
         to_open = list()
         for fname in files:
             try:
@@ -434,7 +660,7 @@ class FileBrowser(wx.GenericDirCtrl):
 
         win = wx.GetApp().GetActiveWindow()
         if win:
-            win.nb.OnDrop(to_open)
+            win.GetNotebook().OnDrop(to_open)
 
       # TODO implement drag and drop from the control to the editor
 #     def OnDragEnd(self, evt):
@@ -531,3 +757,91 @@ class PathMarkConfig(object):
             file_h.write(u"%s=%s\n" % (label, self._pmarks[label]))
         file_h.close()
         return True
+
+#-----------------------------------------------------------------------------#
+# Utilities
+def DuplicatePath(path):
+    """Create a copy of the item at the end of the given path. The item
+    will be created with a name in the form of Dirname_Copy for directories
+    and FileName_Copy.extension for files.
+    @param path: path to duplicate
+    @return: Tuple of (success?, filename OR Error Message)
+
+    """
+    head, tail = os.path.split(path)
+    if os.path.isdir(path):
+        name = util.GetUniqueName(head, tail + "_Copy")
+        copy = shutil.copytree
+    else:
+        tmp = [ part for part in tail.split('.') if len(part) ]
+        if tail.startswith('.'):
+            tmp[0] = "." + tmp[0]
+            if '.' not in tail[1:]:
+                tmp[0] = tmp[0] + "_Copy"
+            else:
+                tmp.insert(-1, "_Copy.")
+
+            tmp = ''.join(tmp)
+        else:
+            tmp = '.'.join(tmp[:-1]) + "_Copy." + tmp[-1]
+
+        if len(tmp) > 1:
+            name = util.GetUniqueName(head, tmp)
+        copy = shutil.copy2
+
+    try:
+        copy(path, name)
+    except Exception, msg:
+        return (False, str(msg))
+
+    return (True, name)
+
+def MakeArchive(path):
+    """Create a Zip archive of the item at the end of the given path
+    @param path: full path to item to archive
+    @return: Tuple of (success?, file name OR Error Message)
+    @rtype: (bool, str)
+    @todo: support for zipping multiple paths
+
+    """
+    dname, fname = os.path.split(path)
+    ok = True
+    name = ''
+    if dname and fname:
+        name = util.GetUniqueName(dname, fname + ".zip")
+        files = list()
+        cwd = os.getcwd()
+        head = dname
+        try:
+            try:
+                os.chdir(dname)
+                if os.path.isdir(path):
+                    for dpath, dname, fnames in os.walk(path):
+                        files.extend([ os.path.join(dpath, fname).\
+                                       replace(head, '', 1).\
+                                       lstrip(os.path.sep) 
+                                       for fname in fnames])
+
+                zfile = zipfile.ZipFile(name, 'w', compression=zipfile.ZIP_DEFLATED)
+                for fname in files:
+                    zfile.write(fname.encode(sys.getfilesystemencoding()))
+            except Exception, msg:
+                ok = False
+                name = str(msg)
+        finally:
+            zfile.close()
+            os.chdir(cwd)
+
+    return (ok, name)
+
+#-----------------------------------------------------------------------------#
+class OpenerThread(threading.Thread):
+    """Job runner thread for opening files with the systems filemanager"""
+    def __init__(self, files):
+        self._files = files
+        threading.Thread.__init__(self)
+
+    def run(self):
+        """Do the work of opeing the files"""
+        for fname in self._files:
+            subprocess.call([FILEMAN_CMD, fname])
