@@ -15,6 +15,13 @@ print_help () {
 if [ "$ARG" = "-po" ]
 then
     env python2.5 mki18n.py -pv --domain=Editra
+    # Copy all .new files to override the originals
+    for fname in $(ls); do
+        if ! [ -z $(echo $fname | grep '.*\.new') ]; then
+            name=$(echo $fname | sed 's/.new//')
+            mv $fname $name
+        fi
+    done
 elif [ "$ARG" = "-mo" ]
 then
     env python2.5 mki18n.py -mv --domain=Editra --moTarget=../../locale
