@@ -3,33 +3,35 @@
 # Purpose: Provides python autocompletion lists and calltips for the editor   #
 # Author: Cody Precord <cprecord@editra.org>                                  #
 # Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: pycomp.py                                                          #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-# SUMMARY:                                                                 #
-#    Provides completion and calltip support for python documents. To      #
-# provide the completion lists and calltips a mix of parsing and           #
-# introspection is used to deduct the requested information.               #
-#                                                                          #
-#--------------------------------------------------------------------------#
+Provides completion and calltip support for python documents. To provide the
+completion lists and calltips a mix of parsing and introspection is used to
+deduct the requested information.
+
+@summary: Python autocompletion support
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__cvsid__ = "$Id: pycomp.py 49905 2007-11-13 13:54:31Z CJP $"
-__revision__ = "$Revision: 49905 $"
+__cvsid__ = "$Id: pycomp.py 52855 2008-03-27 14:53:06Z CJP $"
+__revision__ = "$Revision: 52855 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
-import sys, tokenize, cStringIO, types
-from token import NAME, DEDENT, STRING #, NEWLINE
+import os, sys, tokenize, types
+from token import NAME, DEDENT, STRING, NEWLINE
 import wx
 from wx.py import introspect
 
+# It would be nice to use cStringIO here for the better performance but it
+# doesn't work as uniformly across platfrom as the plain StringIO module. On
+# Linux and python2.4 under all platforms the cStringIO module makes tokens out
+# of each character instead of each actual token which causes the parse to fail.
+from StringIO import StringIO
+
 #--------------------------------------------------------------------------#
 
 class Completer(object):
@@ -65,8 +67,19 @@ class Completer(object):
         """
         try:
             cmpl = PyCompleter()
+
+            # Put the files directory on the path so eval has a better
+            # chance of getting the proper completions
+            fname = self._buffer.GetFileName()
+            if fname:
+                sys.path.insert(0, os.path.dirname(fname))
+
             cmpl.evalsource(self._buffer.GetText(),
                             self._buffer.GetCurrentLine())
+
+            if fname:
+                sys.path.pop(0)
+
             if calltip:
                 return cmpl.get_completions(command + '(', '', calltip)
             else:
@@ -688,8 +701,8 @@ class PyParser:
                    '(' : '()', 'tuple' : '()', 'coerce' : '()'
                  }
 
-        if tokentype == tokenize.STRING or tokentype == tokenize.NUMBER:
-            token = tokentype
+        if tokentype == tokenize.NUMBER or tokentype == tokenize.STRING:
+            return token #token = tokentype
 
         if tokens.has_key(token):
             return tokens[token]
@@ -779,7 +792,7 @@ class PyParser:
 
         """
         self.curline = curline
-        buf = cStringIO.StringIO(text)
+        buf = StringIO(text)
         self.gen = tokenize.generate_tokens(buf.readline)
         self.currentscope = self.scope
 
