@@ -3,32 +3,29 @@
 # Purpose: Creates a small slit panel that holds small controls for searching #
 #          and other actions.                                                 #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: ed_cmdbar.py                                                       #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-# SUMMARY:                                                                 #
-#    This class creates a custom panel that can hide and show different    #
-# controls based an id value. The panel is generally between 24-32 pixels  #
-# in height but can grow to fit the controls inserted in it. The           #
-# the background is painted with a gradient using system defined colors.   #
-#                                                                          #
-#--------------------------------------------------------------------------#
+This class creates a custom panel that can hide and show different controls
+based an id value. The panel is generally between 24-32 pixels in height but
+can grow to fit the controls inserted in it. The the background is painted with
+a gradient using system defined colors.
+
+@summary: The buffers CommandBar control with search/goto line/command entry
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_cmdbar.py 50214 2007-11-24 09:59:05Z CJP $"
-__revision__ = "$Revision: 50214 $"
+__svnid__ = "$Id: ed_cmdbar.py 54280 2008-06-19 02:00:20Z CJP $"
+__revision__ = "$Revision: 54280 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
 import sys
+import cStringIO, zlib
 import glob
 import re
 import wx
@@ -36,57 +33,26 @@ import util
 import ed_glob
 import ed_search
 import ed_event
+import ed_msg
+import eclib.platebtn as platebtn
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
 # Close Button Bitmap
-from wx import ImageFromStream, BitmapFromImage
-import cStringIO, zlib
+from extern.embeddedimage import PyEmbeddedImage
 
-def GetXData():
-    """Returns the raw image data for the close button
-    @return: raw image data
-
-    """
-    return zlib.decompress(
-'x\xda\x011\x02\xce\xfd\x89PNG\r\n\x1a\n\x00\x00\x00\rIHDR\x00\x00\x00\x0e\
-\x00\x00\x00\x0e\x08\x02\x00\x00\x00\x90*\xba\x86\x00\x00\x00\x03sBIT\x08\
-\x08\x08\xdb\xe1O\xe0\x00\x00\x01\xe9IDAT(\x91m\x92\xbdk\x13a\x18\xc0\x9f\
-\xbb\\r\xd7Ks\xb9&\xc6\xd2j\x8f\xc6D\x08\x8a\xa2\xed\xe2G\x87,\x15\xc4\xb5\
-\x8bH\x17-]\x02\x85:\x14\x84\xd0\x82J\x07A\x07\xc1\xff\xc1\xc1I\x04\x07\x11\
-\x05\x15\x1d\x1cJ\x87\xa2\xd5\x96\x92&49\xe9\x99\xf4\xbc7w\xf7\xbe\xef\xf3:\
-\xdcQ:\xf4\x99\x9e\x8f\x1f\xcf\xb7\xf4\xe9\xf1\xc3\xde\xd7\xcfAg/l5\xe08I\
-\x9d\xb2\xd4\xe1\x91\xec\xd5)\xe9\xcd\xad\x1b\x93W.\x9bC\x05.\xfc(&h?\xa6h\
-\xec\xe9\xb5\xbbk\x9b\xbbJ\xd0\xd93\xc6n\xd2\xfd\x16\xddo\xc7\x00c\x00\x80\
-\x9cE\xa6f\x1a\xc6\xe9\x9c\xfasK\x0e[\rP\x12\x9cx\xd2@*=\xbb(\x8d\x8e!g\xc8\
-\x99b\x95rs\x0f\xb8\xaa\xf9\xdd\x03\xa6&\xc5_[>\xecI\x9f\xa9\xa9\x95\x89\xec\
-\xbd\xbab\x95\x14\xab\x94\x9f_V+\x13\x85\xd9E\x1f\xa9\xc2)\x00(\x00 \x04\x13\
-\x10\x90\xf7\xaf\x92\xe3\x15Y\xd3\xf3\xf3\xcb\x00 k:\xfa\xc4y\xfb\xf20\x97\
-\x02\x00\xd0%\xc1\xae\xc36\xb6\x83\xe6\xfd\x93+\xcfdM\x07\x00\xf4I\xa3\xbe\
-\xe0o\xfd\x00\x00\x9e\x1b\x8dQ\xf6\x8f0/\xa0\xae/\x05\xec\xe8\x9a0\xe4\xcc\
-\xf59"w\\\xc1\xb8\x0c\x00"d\x88"u\xf6\xfc\xc8\xea\xf3\xa8.\xfaD\xd6\xf4\xf1\
-\'/\xf4s\x17\x00\x00(\x17!\x8d\xc7B\xa4\xd9\xdbw#ng\xa9\xb6\xb3T\x8b\xe8\xfc\
-\x9d9\x00\x10,\x00\xc4\xc4LF+_\xbaHl\xdb\xfb\xf2!Y,\xb7\x9e>"\x9b\x1b\xb4\
-\xd3\xf6\xd6\xbe\xcb\xb9Bc\xb5.3\xaa\r\x19\x9df[z}}r\xaaz\xad\xff\xc7>\xb0\
-\x9b\xcc\x8d\xcf\xc3\x11#%\x91J\x0e\x9a\x99\xf4\xe0\xc0z\xb3\xa7\xe8V\xd1u\
-\x1c#sB\xcb\xa4\x01@`\x185\x17\xd5e\x88\t\xce\xdc^\x1fr\xc3\x8aY\x9d\xfe\xf5\
-\xf1\x1dY\xff\xe6m\xff>\xf6]\xd2g\xca\xbaU4\xab\xd3\xff\x01\xe3\xf6\xf0\x91\
-\xbc\xe0^J\x00\x00\x00\x00IEND\xaeB`\x82\x7fU\x05\xed' )
-
-def GetXBitmap():
-    """Returns a bitmap version of the close button
-    @return: bitmap of close button
-
-    """
-    return BitmapFromImage(GetXImage())
-
-def GetXImage():
-    """Returns an image version of the close button
-    @return: image of close button
-
-    """
-    stream = cStringIO.StringIO(GetXData())
-    return ImageFromStream(stream)
+XButton = PyEmbeddedImage(
+    "iVBORw0KGgoAAAANSUhEUgAAAA4AAAAOCAIAAACQKrqGAAAAA3NCSVQICAjb4U/gAAAB6UlE"
+    "QVQokW2SvWsTYRjAn7tcctdLc7kmxtJqj8ZECIqi7eJHhywVxLWLSBctXQKFOhSE0IJKB0EH"
+    "wf/BwUkEBxEFFR0cSoei1ZaSJjQ56Zn0vDd3977v8zrcUTr0mZ6PH8+39Onxw97Xz0FnL2w1"
+    "4DhJnbLU4ZHs1Snpza0bk1cum0MFLvwoJmg/pmjs6bW7a5u7StDZM8Zu0v0W3W/HAGMAgJxF"
+    "pmYaxumc+nNLDlsNUBKceNJAKj27KI2OIWfImWKVcnMPuKr53QOmJsVfWz7sSZ+pqZWJ7L26"
+    "YpUUq5SfX1YrE4XZRR+pwikAKAAgBBMQkPevkuMVWdPz88sAIGs6+sR5+/IwlwIA0CXBrsM2"
+    "toPm/ZMrz2RNBwD0SaO+4G/9AACeG41R9o8wL6CuLwXs6Jow5Mz1OSJ3XMG4DAAiZIgidfb8"
+    "yOrzqC76RNb08Scv9HMXAAAoFyGNx0Kk2dt3I25nqbazVIvo/J05ABAsAMTETEYrX7pIbNv7"
+    "8iFZLLeePiKbG7TT9ta+y7lCY7UuM6oNGZ1mW3p9fXKqeq3/xz6wm8yNz8MRIyWRSg6amfTg"
+    "wHqzp+hW0XUcI3NCy6QBQGAYNRfVZYgJztxeH3LDilmd/vXxHVn/5m3/PvZd0mfKulU0q9P/"
+    "AeP28JG84F5KAAAAAElFTkSuQmCC")
 
 #-----------------------------------------------------------------------------#
 # Globals
@@ -95,11 +61,8 @@ ID_SEARCH_CTRL = wx.NewId()
 ID_SEARCH_NEXT = wx.NewId()
 ID_SEARCH_PRE = wx.NewId()
 ID_MATCH_CASE = wx.NewId()
-ID_FIND_LBL = wx.NewId()
 ID_LINE_CTRL = wx.NewId()
-ID_GOTO_LBL = wx.NewId()
 ID_CMD_CTRL = wx.NewId()
-ID_CMD_LBL = wx.NewId()
 
 #-----------------------------------------------------------------------------#
 
@@ -134,7 +97,7 @@ class CommandBar(wx.Panel):
             bstyle = wx.NO_BORDER
 
         self.close_b = wx.BitmapButton(self, ID_CLOSE_BUTTON, \
-                                       GetXBitmap(), style=bstyle)
+                                       XButton.GetBitmap(), style=bstyle)
         self._sizers['h_sizer'].Add(self.close_b, 0, wx.ALIGN_CENTER_VERTICAL)
         self._sizers['h_sizer'].Add((12, 12))
         v_sizer.Add((2, 2))
@@ -143,7 +106,12 @@ class CommandBar(wx.Panel):
         self.SetAutoLayout(True)
 
         # Bind Events
-        self.Bind(wx.EVT_PAINT, self.OnPaint)
+        ed_msg.Subscribe(self.OnThemeChange, ed_msg.EDMSG_THEME_CHANGED)
+
+        # Don't paint the gradient on gtk due to transparency
+        # issues with some controls
+        if wx.Platform != '__WXGTK__':
+            self.Bind(wx.EVT_PAINT, self.OnPaint)
         self.Bind(wx.EVT_BUTTON, self.OnButton)
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
 
@@ -187,7 +155,7 @@ class CommandBar(wx.Panel):
                             size=(100, 20))
         v_sizer.Add(linectrl, 0, wx.ALIGN_CENTER_VERTICAL)
         v_sizer.Add((4, 4))
-        go_lbl = wx.StaticText(self, ID_GOTO_LBL, _("Goto Line") + ": ")
+        go_lbl = wx.StaticText(self, label=_("Goto Line") + ": ")
         if wx.Platform == '__WXMAC__':
             go_lbl.SetFont(wx.SMALL_FONT)
         h_sizer.AddMany([(go_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
@@ -210,7 +178,7 @@ class CommandBar(wx.Panel):
         cmdctrl = CommandExecuter(self, ID_CMD_CTRL, size=(150, 20))
         v_sizer.Add(cmdctrl, 0, wx.ALIGN_CENTER_VERTICAL)
         v_sizer.Add((4, 4))
-        cmd_lbl = wx.StaticText(self, ID_CMD_LBL, _("Command") + ": ")
+        cmd_lbl = wx.StaticText(self, label=_("Command") + ": ")
         if wx.Platform == '__WXMAC__':
             cmd_lbl.SetFont(wx.SMALL_FONT)
         h_sizer.AddMany([(cmd_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
@@ -236,40 +204,36 @@ class CommandBar(wx.Panel):
         if wx.Platform == '__WXGTK__':
             spacer = (4, 4)
         v_sizer.Add(spacer)
-        search = ed_search.EdSearchCtrl(self, ID_SEARCH_CTRL, 
-                                         menulen=5, size=(180, -1))
+        search = ed_search.EdSearchCtrl(self, ID_SEARCH_CTRL,
+                                        menulen=5, size=(180, 20))
         v_sizer.Add(search, 0, wx.ALIGN_CENTER_VERTICAL)
         v_sizer.Add((4, 4))
-        f_lbl = wx.StaticText(self, ID_FIND_LBL, _("Find") + u": ")
+        f_lbl = wx.StaticText(self, label=_("Find") + u": ")
         ctrl_sizer = wx.BoxSizer(wx.HORIZONTAL)
         t_bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_DOWN), wx.ART_MENU)
-        next_btn = wx.BitmapButton(self, ID_SEARCH_NEXT, 
-                                   t_bmp, style=wx.NO_BORDER)
-        nlbl = wx.StaticText(self, label=_("Next"))
+        next_btn = platebtn.PlateButton(self, ID_SEARCH_NEXT, _("Next"),
+                                        t_bmp, style=platebtn.PB_STYLE_NOBG)
 
         t_bmp = wx.ArtProvider.GetBitmap(str(ed_glob.ID_UP), wx.ART_MENU)
-        pre_btn = wx.BitmapButton(self, ID_SEARCH_PRE, 
-                                  t_bmp, style=wx.NO_BORDER)
-        plbl = wx.StaticText(self, label=_("Previous"))
+        pre_btn = platebtn.PlateButton(self, ID_SEARCH_PRE, _("Previous"),
+                                       t_bmp, style=platebtn.PB_STYLE_NOBG)
 
         match_case = wx.CheckBox(self, ID_MATCH_CASE, _("Match Case"))
         match_case.SetValue(search.IsMatchCase())
         if wx.Platform == '__WXMAC__':
             t_sizer.Add((5, 5))
-            for win in [f_lbl, match_case, nlbl, plbl]:
+            for win in [f_lbl, match_case, next_btn, pre_btn]:
                 win.SetFont(wx.SMALL_FONT)
 
-        ctrl_sizer.AddMany([(10, 0), (next_btn, 0, wx.ALIGN_CENTER_VERTICAL), 
-                            ((3, 3)), (nlbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                            ((10, 0)), (pre_btn, 0, wx.ALIGN_CENTER_VERTICAL), 
-                            ((3, 3)), (plbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                            ((10, 0)), 
+        ctrl_sizer.AddMany([(10, 0), (next_btn, 0, wx.ALIGN_CENTER_VERTICAL),
+                            ((5, 0)), (pre_btn, 0, wx.ALIGN_CENTER_VERTICAL),
+                            ((8, 0)),
                             (match_case, 0, wx.ALIGN_CENTER_VERTICAL)])
 
         t_sizer.Add(ctrl_sizer, 0, wx.ALIGN_CENTER_VERTICAL)
 
         h_sizer.AddMany([(f_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                         ((5, 5)), (v_sizer, 0, wx.ALIGN_CENTER_VERTICAL), 
+                         ((5, 5)), (v_sizer, 0, wx.ALIGN_CENTER_VERTICAL),
                          (t_sizer, 0, wx.ALIGN_CENTER_VERTICAL)])
         self._sizers['search'] = h_sizer
         self._sizers['h_sizer'].Add(h_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL)
@@ -306,13 +270,7 @@ class CommandBar(wx.Panel):
         elif e_id in [ID_SEARCH_NEXT, ID_SEARCH_PRE]:
             search = self.FindWindowById(ID_SEARCH_CTRL)
             if search != None:
-                evt = wx.KeyEvent(wx.wxEVT_KEY_UP)
-                evt.m_keyCode = wx.WXK_RETURN
-                if e_id == ID_SEARCH_PRE:
-                    evt.m_shiftDown = True
-                else:
-                    evt.m_shiftDown = False
-                wx.PostEvent(search, evt)
+                search.DoSearch(e_id == ID_SEARCH_NEXT)
         else:
             evt.Skip()
 
@@ -325,18 +283,25 @@ class CommandBar(wx.Panel):
         dc = wx.PaintDC(self)
         gc = wx.GraphicsContext.Create(dc)
         col1 = wx.SystemSettings_GetColour(wx.SYS_COLOUR_3DFACE)
-        col2 = util.AdjustColour(col1, 50)
-        col1 = util.AdjustColour(col1, -60)
+        col2 = util.AdjustColour(col1, 40)
+        col1 = util.AdjustColour(col1, -15)
         grad = gc.CreateLinearGradientBrush(0, 1, 0, 29, col2, col1)
         rect = self.GetClientRect()
 
-        pen_col = tuple([min(190, x) for x in util.AdjustColour(col1, -60)])
+        pen_col = tuple([min(190, x) for x in util.AdjustColour(col1, 20)])
         gc.SetPen(gc.CreatePen(wx.Pen(pen_col, 1)))
         gc.SetBrush(grad)
         gc.DrawRectangle(0, 1, rect.width - 0.5, rect.height - 0.5)
 
         evt.Skip()
 
+    def OnThemeChange(self, msg):
+        """Update icons when the theme has changed
+        @param msg: Message Object
+
+        """
+        self.UpdateIcons()
+
     def Show(self, id_=0):
         """Shows the control and installs it in the parents
         sizer if not installed already.
@@ -359,7 +324,7 @@ class CommandBar(wx.Panel):
                 ctrl = self.InstallCtrl(id_)
 
             # First Hide everything
-            for kid in (list(self._sizers['search'].GetChildren()) + 
+            for kid in (list(self._sizers['search'].GetChildren()) +
                         list(self._sizers['goto'].GetChildren()) +
                         list(self._sizers['cmd'].GetChildren())):
                 kid.Show(False)
@@ -367,6 +332,7 @@ class CommandBar(wx.Panel):
             if id_ == ID_SEARCH_CTRL:
                 for kid in self._sizers['search'].GetChildren():
                     kid.Show(True)
+                self.FindWindowById(ID_SEARCH_CTRL).AutoSetQuery()
                 self._sizers['search'].Layout()
             elif id_ == ID_LINE_CTRL:
                 for kid in self._sizers['goto'].GetChildren():
@@ -406,7 +372,7 @@ class CommandBar(wx.Panel):
 class CommandExecuter(wx.SearchCtrl):
     """Part of the Vi emulation, opens a minibuffer to execute EX commands.
     @note: based on search ctrl so we get the nice roudned edges on wxmac.
-    
+
     """
     RE_GO_BUFFER = re.compile('[0-9]*[nN]{1,1}')
     RE_GO_WIN = re.compile('[0-9]*n[wW]{1,1}')
@@ -415,7 +381,7 @@ class CommandExecuter(wx.SearchCtrl):
 
     def __init__(self, parent, id_, size=wx.DefaultSize):
         """Initializes the CommandExecuter"""
-        wx.SearchCtrl.__init__(self, parent, id_, size=size, 
+        wx.SearchCtrl.__init__(self, parent, id_, size=size,
                                style=wx.TE_PROCESS_ENTER|wx.WANTS_CHARS)
 
         # Attributes
@@ -503,7 +469,7 @@ class CommandExecuter(wx.SearchCtrl):
         else:
             self.Clear()
             wx.Bell()
-            
+
     def CommandPush(self, cmd):
         """Push a command to the stack popping as necessary to
         keep stack size less than MAX (currently 25 commands).
@@ -550,7 +516,7 @@ class CommandExecuter(wx.SearchCtrl):
             cmd = 'wq'
 
         if cmd.startswith(u'w'):
-            frame.OnSave(wx.MenuEvent(wx.wxEVT_COMMAND_MENU_SELECTED, 
+            frame.OnSave(wx.MenuEvent(wx.wxEVT_COMMAND_MENU_SELECTED,
                                                      ed_glob.ID_SAVE))
             if self.RE_WGO_BUFFER.match(cmd):
                 self.GoBuffer(cmd[1:])
@@ -707,7 +673,7 @@ class CommandExecuter(wx.SearchCtrl):
             cstr = 'e '
         else:
             return
-            
+
         cmd = cmd.replace(cstr, u'', 1).strip()
         paths = self.GetPaths(cmd, cstr == 'e ')
         self._popup.SetChoices(paths)
@@ -724,7 +690,7 @@ class CommandExecuter(wx.SearchCtrl):
     def OnEnter(self, evt):
         """Get the currently entered command string and execute it.
         @postcondition: ctrl is cleared and command is executed
-        
+
         """
         if self._popup.IsShown():
             psel = self._popup.GetSelection()
@@ -825,13 +791,13 @@ class CommandExecuter(wx.SearchCtrl):
         @postcondition: Editor begins exit, confirming file saves
 
         """
-        wx.PostEvent(self.GetTopLevelParent(), 
+        wx.PostEvent(self.GetTopLevelParent(),
                      wx.CloseEvent(wx.wxEVT_CLOSE_WINDOW))
 
     def SetValue(self, value):
         """Overrides the controls default function to allow for automatic
         resizing of the control when text is added.
-        @param val: string to set value of control to
+        @param value: string to set value of control to
 
         """
         wx.SearchCtrl.SetValue(self, value)
@@ -922,10 +888,10 @@ class PopupList(wx.Frame):
         wx.Frame.__init__(self, parent, pos=pos, style=style)
 
         # Attributes
-        self._list = wx.ListBox(self, choices=choices, 
-                                style=wx.LC_REPORT | wx.LC_SINGLE_SEL | 
+        self._list = wx.ListBox(self, choices=choices,
+                                style=wx.LC_REPORT | wx.LC_SINGLE_SEL |
                                       wx.LC_NO_HEADER | wx.NO_BORDER)
-        
+
         # Layout
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(self._list, 1, wx.EXPAND)
@@ -949,7 +915,7 @@ class PopupList(wx.Frame):
 
         """
         val = self._list.GetStringSelection()
-        evt = ed_event.NotificationEvent(ed_event.edEVT_NOTIFY, 
+        evt = ed_event.NotificationEvent(ed_event.edEVT_NOTIFY,
                                           self.GetId(), val, self._list)
         wx.PostEvent(self.GetParent(), evt)
         self.ActivateParent()
@@ -1020,7 +986,7 @@ class PopupList(wx.Frame):
         """Handle a selection in list by posting the result to
         the parent.
         @param evt: Event that called this handler
-        
+
         """
         self.__PostEvent()
 
@@ -1079,7 +1045,7 @@ class PopupList(wx.Frame):
                 self._list.SetSelection(0)
                 self.ActivateParent()
         else:
-            matches = [item for item in self._list.GetItems() 
+            matches = [item for item in self._list.GetItems()
                        if item.startswith(prefix) ]
             if len(matches):
                 self._list.SetStringSelection(sorted(matches)[0])
@@ -1096,9 +1062,9 @@ class PopupWinList(wx.PopupWindow):
 
         # Attributes
         self._list = wx.ListBox(self, choices=choices, pos=(0, 0),
-                                style=wx.LC_REPORT | wx.LC_SINGLE_SEL | 
+                                style=wx.LC_REPORT | wx.LC_SINGLE_SEL |
                                       wx.LC_NO_HEADER | wx.NO_BORDER)
-        
+
         # Layout
         sizer = wx.BoxSizer(wx.HORIZONTAL)
         sizer.Add(self._list, 0, wx.EXPAND)
@@ -1152,7 +1118,7 @@ class PopupWinList(wx.PopupWindow):
             if len(self._list.GetStrings()):
                 self._list.SetSelection(0)
         else:
-            matches = [item for item in self._list.GetItems() 
+            matches = [item for item in self._list.GetItems()
                        if item.startswith(prefix) ]
             if len(matches):
                 self._list.SetStringSelection(sorted(matches)[0])
@@ -1180,4 +1146,4 @@ class PopupWinList(wx.PopupWindow):
         self._list.SetInitialSize()
         self.SetInitialSize()
 
-        return res
\ No newline at end of file
+        return res
