@@ -2,44 +2,45 @@
 # Name: ed_pages.py                                                           #
 # Purpose: The main editor notebook                                           #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: ed_pages.py                                                        #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-#                                                                          #
-# SUMMARY:                                                                 #
-# This file defines the notebook for containing the text controls for      #
-# for editing text in Editra. The note book is a custom sublclass of       #
-# FlatNotebook that allow for automatic page images and drag and dropping  #
-# of tabs between open editor windows. The notebook is also primarly in    #
-# charge of opening files that are requested by the user and setting up the#
-# text control to use them. For more information on the text controls used #
-# in the notebook see ed_stc.py                                            #
-#                                                                          #
-#--------------------------------------------------------------------------#
+This file defines the notebook for containing the text controls for
+for editing text in Editra. The note book is a custom sublclass of
+FlatNotebook that allow for automatic page images and drag and dropping
+of tabs between open editor windows. The notebook is also primarly in
+charge of opening files that are requested by the user and setting up the
+text control to use them. For more information on the text controls used
+in the notebook see ed_stc.py
+
+@summary: Editra's main notebook class
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_pages.py 50254 2007-11-26 13:03:00Z CJP $"
-__revision__ = "$Revision: 50254 $"
+__svnid__ = "$Id: ed_pages.py 54281 2008-06-19 02:17:46Z CJP $"
+__revision__ = "$Revision: 54281 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
 import glob
 import wx
+
+# Editra Libraries
 import ed_glob
 from profiler import Profile_Get
 import ed_stc
 import syntax.synglob as synglob
+import syntax.syntax as syntax
 import ed_search
 import util
 import doctools
+import ed_msg
+import ed_txt
+import ed_mdlg
 from extern import flatnotebook as FNB
 
 #--------------------------------------------------------------------------#
@@ -47,6 +48,7 @@ from extern import flatnotebook as FNB
 
 _ = wx.GetTranslation
 #--------------------------------------------------------------------------#
+
 class EdPages(FNB.FlatNotebook):
     """Editras editor buffer botebook
     @todo: allow for tab styles to be configurable (maybe)
@@ -58,9 +60,9 @@ class EdPages(FNB.FlatNotebook):
         @param id_num: this notebooks id
 
         """
-        FNB.FlatNotebook.__init__(self, parent, id_num, 
+        FNB.FlatNotebook.__init__(self, parent, id_num,
                                   style=FNB.FNB_FF2 |
-                                        FNB.FNB_X_ON_TAB | 
+                                        FNB.FNB_X_ON_TAB |
                                         FNB.FNB_SMART_TABS |
                                         FNB.FNB_BACKGROUND_GRADIENT |
                                         FNB.FNB_DROPDOWN_TABS_LIST |
@@ -71,11 +73,12 @@ class EdPages(FNB.FlatNotebook):
         self.LOG = wx.GetApp().GetLog()
         self.FindService = ed_search.TextFinder(self, self.GetCurrentCtrl)
         self.DocMgr = doctools.DocPositionMgr(ed_glob.CONFIG['CACHE_DIR'] + \
-                                              util.GetPathChar() + u'positions')
+                                              os.sep + u'positions')
         self.pg_num = -1              # Track new pages (aka untitled docs)
         self.control = None
         self.frame = self.GetTopLevelParent() # MainWindow
         self._index = dict()          # image list index
+        self._ses_load = False
 
         # Set Additional Style Parameters
         self.SetNonActiveTabTextColour(wx.Colour(102, 102, 102))
@@ -96,6 +99,7 @@ class EdPages(FNB.FlatNotebook):
         self.Bind(wx.stc.EVT_STC_MODIFIED, self.OnUpdatePageText)
         self._pages.Bind(wx.EVT_LEFT_UP, self.OnLeftUp)
         self.Bind(wx.EVT_IDLE, self.OnIdle)
+        ed_msg.Subscribe(self.OnThemeChanged, ed_msg.EDMSG_THEME_CHANGED)
 
         # Add a blank page
         self.NewPage()
@@ -106,8 +110,8 @@ class EdPages(FNB.FlatNotebook):
     def _NeedOpen(self, path):
         """Check if a file needs to be opened. If the file is already open in
         the notebook a dialog will be opened to ask if the user wants to reopen
-        the file again. If the file is not open and exists or the user chooses 
-        to reopen the file again the function will return True else it will 
+        the file again. If the file is not open and exists or the user chooses
+        to reopen the file again the function will return True else it will
         return False.
         @param path: file to check for
 
@@ -117,7 +121,7 @@ class EdPages(FNB.FlatNotebook):
             mdlg = wx.MessageDialog(self,
                                     _("File is already open in an existing "
                                       "page.\nDo you wish to open it again?"),
-                                    _("Open File") + u"?", 
+                                    _("Open File") + u"?",
                                     wx.YES_NO | wx.NO_DEFAULT | \
                                     wx.ICON_INFORMATION)
             result = mdlg.ShowModal()
@@ -127,7 +131,6 @@ class EdPages(FNB.FlatNotebook):
                     ctrl = self.GetPage(page)
                     if path == ctrl.GetFileName():
                         self.SetSelection(page)
-                        self.ChangePage(page)
                         break
         elif os.path.exists(path) and not os.path.isfile(path):
             result = wx.ID_NO
@@ -166,12 +169,15 @@ class EdPages(FNB.FlatNotebook):
                         single blank page is opened.
 
         """
+        self._ses_load = True
         files = Profile_Get('LAST_SESSION')
         if files is not None:
             for fname in files:
                 if os.path.exists(fname) and os.access(fname, os.R_OK):
-                    self.OpenPage(os.path.dirname(fname), 
+                    self.OpenPage(os.path.dirname(fname),
                                   os.path.basename(fname))
+                    self.Update() # Give feedback as files are loaded
+        self._ses_load = False
 
         if self.GetPageCount() == 0:
             self.NewPage()
@@ -184,11 +190,24 @@ class EdPages(FNB.FlatNotebook):
         self.GetTopLevelParent().Freeze()
         self.pg_num += 1
         self.control = ed_stc.EditraStc(self, wx.ID_ANY)
-        self.LOG("[nb_evt] Page Creation ID: %d" % self.control.GetId())
+        self.LOG("[ed_pages][evt] New Page Created ID: %d" % self.control.GetId())
         self.AddPage(self.control, u"Untitled - %d" % self.pg_num)
         self.SetPageImage(self.GetSelection(), str(self.control.GetLangId()))
+
+        # Set the control up the the preferred default lexer
+        dlexer = Profile_Get('DEFAULT_LEX', 'str', 'Plain Text')
+        ext_reg = syntax.ExtensionRegister()
+        ext_lst = ext_reg.get(dlexer, ['txt', ])
+        self.control.FindLexer(ext_lst[0])
         self.GetTopLevelParent().Thaw()
 
+    def OnThemeChanged(self, msg):
+        """Update icons when the theme has changed
+        @param msg: Message Object
+
+        """
+        self.UpdateAllImages()
+
     def OpenPage(self, path, filename):
         """Open a File Inside of a New Page
         @param path: files base path
@@ -211,34 +230,68 @@ class EdPages(FNB.FlatNotebook):
                 control.Hide()
             else:
                 new_pg = False
+                control = self.control
         else:
             control = ed_stc.EditraStc(self, wx.ID_ANY)
             control.Hide()
 
         # Open file and get contents
-        err = False
-        in_txt = u''
-        enc = u'utf-8'
+        result = False
         if os.path.exists(path2file):
             try:
-                in_txt, enc = util.GetDecodedText(path2file)
-            except (UnicodeDecodeError, IOError, OSError), msg:
-                self.LOG(("[ed_pages][err] Failed to open file %s\n"
-                          "[ed_pages][err] %s") % (path2file, msg))
+                result = control.LoadFile(path2file)
+            except Exception, msg:
+                self.LOG("[ed_pages][err] Failed to open file %s\n" % path2file)
+                self.LOG("[ed_pages][err] %s" % msg)
 
                 # File could not be opened/read give up
-                err = wx.MessageDialog(self, _("Editra could not properly "
-                                               "open %s\n") \
-                                       % path2file, _("Error Opening File"),
-                                       style=wx.OK | wx.CENTER | wx.ICON_ERROR)
-                err.ShowModal()
-                err.Destroy()
+                # Don't raise a dialog during a session load error as if the
+                # dialog is shown before the mainwindow is ready it can cause
+                # the app to freeze.
+                if not self._ses_load:
+                    ed_mdlg.OpenErrorDlg(self, path2file, msg)
+                control.GetDocument().ClearLastError()
+                control.SetFileName('') # Reset the file name
 
                 if new_pg:
                     control.Destroy()
 
                 self.GetTopLevelParent().Thaw()
                 return
+        else:
+            control.SetFileName(path2file)
+            result = True
+
+        # Check if there was encoding errors
+        if not result:
+            doc = control.GetDocument()
+            doc.ClearLastError()
+            enc = doc.GetEncoding()
+            enc1 = doc.GetMagic()
+            if enc1 is None:
+                enc1 = ed_txt.DEFAULT_ENCODING
+
+            if not self._ses_load:
+                errmap = dict(encoding1=enc1, encoding2=enc)
+                dlg = wx.MessageDialog(self,
+                   _("The document could not be decoded with %(encoding1)s."
+                     "\n\nWould you like to open it as %(encoding2)s instead?")
+                   % errmap, _("Encoding Error"),
+                   style=wx.YES_NO|wx.ICON_WARNING)
+                dlg.CenterOnParent()
+                result = dlg.ShowModal()
+
+        # Don't want to open it in another encoding
+        if result == wx.ID_NO or not result:
+            if new_pg:
+                control.Destroy()
+            else:
+                control.SetText('')
+                control.SetDocument(ed_txt.EdFile())
+                control.SetSavePoint()
+
+            self.GetTopLevelParent().Thaw()
+            return
 
         # Put control into page an place page in notebook
         if new_pg:
@@ -246,17 +299,15 @@ class EdPages(FNB.FlatNotebook):
             self.control = control
 
         # Pass directory and file name info to control object to save reference
-        self.control.SetText(in_txt, enc)
-        self.control.SetFileName(path2file)
         self.control.SetModTime(util.GetFileModTime(path2file))
         self.frame.AddFileToHistory(path2file)
         if new_pg:
             self.AddPage(self.control, filename)
 
-        self.frame.SetTitle("%s - file://%s" % (filename, 
+        self.frame.SetTitle("%s - file://%s" % (filename,
                                                 self.control.GetFileName()))
         self.SetPageText(self.GetSelection(), filename)
-        self.LOG("[nb_evt] Opened Page: ID = %d" % self.GetSelection())
+        self.LOG("[ed_pages][evt] Opened Page: %s" % filename)
 
         # Setup Document
         self.control.FindLexer()
@@ -272,6 +323,7 @@ class EdPages(FNB.FlatNotebook):
         # Refocus on selected page
         self.GoCurrentPage()
         self.GetTopLevelParent().Thaw()
+        ed_msg.PostMessage(ed_msg.EDMSG_FILE_OPENED, self.control.GetFileName())
 
     def GoCurrentPage(self):
         """Move Focus to Currently Selected Page.
@@ -282,7 +334,7 @@ class EdPages(FNB.FlatNotebook):
         if current_page < 0:
             return current_page
 
-        self.LOG("[nb_info] Current Page = %d" % current_page)
+        self.LOG("[ed_pages][info] Current Page = %d" % current_page)
 
         control = self.GetPage(current_page)
         control.SetFocus()
@@ -339,22 +391,24 @@ class EdPages(FNB.FlatNotebook):
         # Check file properties and make a "clean" list of file(s) to open
         valid_files = list()
         for fname in files:
-            self.LOG("[fdt_evt] File(s) Dropped: %s" % fname)
+            self.LOG("[ed_pages][evt] File(s) Dropped: %s" % fname)
             if not os.path.exists(fname):
                 self.frame.PushStatusText(_("Invalid file: %s") % fname, \
                                           ed_glob.SB_INFO)
             elif os.path.isdir(fname):
                 dcnt = glob.glob(os.path.join(fname, '*'))
                 dcnt = util.FilterFiles(dcnt)
+                dlg = None
                 if not len(dcnt):
-                    dlg = wx.MessageDialog(self, 
+                    dlg = wx.MessageDialog(self,
                                            _("There are no files that Editra"
                                              " can open in %s") % fname,
                                            _("No Valid Files to Open"),
                                            style=wx.OK | wx.CENTER | \
                                                  wx.ICON_INFORMATION)
-                else:
-                    dlg = wx.MessageDialog(self, 
+                elif len(dcnt) > 5:
+                    # Warn when the folder contains many files
+                    dlg = wx.MessageDialog(self,
                                            _("Do you wish to open all %d files"
                                              " in this directory?\n\nWarning"
                                              " opening many files at once may"
@@ -363,8 +417,12 @@ class EdPages(FNB.FlatNotebook):
                                            _("Open Directory?"),
                                            style=wx.YES | wx.NO | \
                                                  wx.ICON_INFORMATION)
-                result = dlg.ShowModal()
-                dlg.Destroy()
+                if dlg is not None:
+                    result = dlg.ShowModal()
+                    dlg.Destroy()
+                else:
+                    result = wx.ID_YES
+
                 if result == wx.ID_YES:
                     valid_files.extend(dcnt)
                 else:
@@ -397,9 +455,9 @@ class EdPages(FNB.FlatNotebook):
                 wx.CallAfter(AskToReload, self, cfile)
             else:
                 return False
-                        
+
     def OnLeftUp(self, evt):
-        """Traps clicks sent to page close buttons and 
+        """Traps clicks sent to page close buttons and
         redirects the action to the ClosePage function
         @param evt: Event that called this handler
         @type evt: wx.MouseEvent
@@ -415,7 +473,7 @@ class EdPages(FNB.FlatNotebook):
         elif cord == FNB.FNB_TAB_X:
             # Make sure that the button was pressed before
             if self._pages._nTabXButtonStatus != FNB.FNB_BTN_PRESSED:
-                return 
+                return
             self._pages._nTabXButtonStatus = FNB.FNB_BTN_HOVER
             self.ClosePage()
         else:
@@ -428,7 +486,10 @@ class EdPages(FNB.FlatNotebook):
 
         """
         evt.Skip()
-        self.LOG("[nb_evt] Page Changed to %d" % evt.GetSelection())
+        pages = (evt.GetOldSelection(), evt.GetSelection())
+        self.LOG("[ed_pages][evt] Control Changing from Page: "
+                  "%d to Page: %d\n" % pages)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGING, (self,) + pages)
 
     def ChangePage(self, pgid):
         """Change the page and focus to the the given page id
@@ -442,7 +503,6 @@ class EdPages(FNB.FlatNotebook):
 
         if fname == "":
             fname = self.GetPageText(pgid)
-
         self.frame.SetTitle("%s - file://%s" % (util.GetFileName(fname), fname))
 
     def OnPageChanged(self, evt):
@@ -451,12 +511,15 @@ class EdPages(FNB.FlatNotebook):
         @type evt: wx.lib.flatnotebook.EVT_FLATNOTEBOOK_PAGE_CHANGED
 
         """
-        self.ChangePage(evt.GetSelection())
-        self.LOG(("[nb_evt] Control Changing from Page: %d to Page: %d\n"
-                  "[nb_info] It has file named: %s" % (evt.GetOldSelection(), 
-                                                       evt.GetSelection(), 
-                                                       self.control.GetFileName())))
+        cpage = evt.GetSelection()
+        self.ChangePage(cpage)
         evt.Skip()
+        self.LOG("[ed_pages][evt] Page Changed to %d" % cpage)
+        self.LOG("[ed_pages][info] It has file named: %s" % \
+                 self.control.GetFileName())
+        self.control.PostPositionEvent()
+        self.EnsureVisible(cpage)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CHANGED, (self, cpage))
 
     def OnPageClosing(self, evt):
         """Checks page status to flag warnings before closing
@@ -464,11 +527,13 @@ class EdPages(FNB.FlatNotebook):
         @type evt: wx.lib.flatnotebook.EVT_FLATNOTEBOOK_PAGE_CLOSING
 
         """
-        self.LOG("[nb_evt] Closing Page: #%d" % self.GetSelection())
+        self.LOG("[ed_pages][evt] Closing Page: #%d" % self.GetSelection())
         page = self.GetCurrentPage()
         if len(page.GetFileName()) > 1:
             self.DocMgr.AddRecord([page.GetFileName(), page.GetCurrentPos()])
         evt.Skip()
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSING,
+                           (self, self.GetSelection()))
 
     def OnPageClosed(self, evt):
         """Handles Paged Closed Event
@@ -476,8 +541,11 @@ class EdPages(FNB.FlatNotebook):
         @type evt: wx.lib.flatnotebook.EVT_FLATNOTEBOOK_PAGE_CLOSED
 
         """
-        self.LOG("[nb_evt] Closed Page: #%d" % self.GetSelection())
+        cpage = self.GetSelection()
         evt.Skip()
+        self.LOG("[ed_pages][evt] Closed Page: #%d" % cpage)
+        ed_msg.PostMessage(ed_msg.EDMSG_UI_NB_CLOSED, (self, cpage))
+
     #---- End Event Handlers ----#
 
     def CloseAllPages(self):
@@ -488,9 +556,8 @@ class EdPages(FNB.FlatNotebook):
         for page in xrange(self.GetPageCount()):
             result = self.ClosePage()
             if result == wx.ID_CANCEL:
-                self.LOG("[nb][closeall] Canceled on page %d" % page)
                 break
-            
+
     def ClosePage(self):
         """Closes Currently Selected Page
         @postcondition: currently selected page is closed
@@ -538,6 +605,15 @@ class EdPages(FNB.FlatNotebook):
                                               GetBitmap(lang_id, wx.ART_MENU))
         FNB.FlatNotebook.SetPageImage(self, pg_num, self._index[lang_id])
 
+    def SetSelection(self, pgnum):
+        """Set the currently selected page
+        @param pgnum: page number index
+
+        """
+        csel = self.GetSelection()
+        FNB.FlatNotebook.SetSelection(self, pgnum)
+        self.ChangePage(pgnum)
+
     def UpdateAllImages(self):
         """Reload and Reset all images in the notebook pages and
         the corresponding imagelist to match those of the current theme
@@ -560,29 +636,30 @@ class EdPages(FNB.FlatNotebook):
         @postcondition: page image is updated to reflect any changes in ctrl
 
         """
-        pg_num = self.GetSelection()
-        ftype = util.GetExtension(self.control.GetFileName())
-        ftype = ftype[-1].upper()
-        self.LOG("[nb_info] Updating Page Image: Page %d" % pg_num)
-        self.SetPageImage(pg_num, str(self.control.GetLangId()))
+        self.SetPageImage(self.GetSelection(), str(self.control.GetLangId()))
 
     def OnUpdatePageText(self, evt):
         """Update the title text of the current page
         @param evt: event that called this handler
-        @type evt: stc.EVT_STC_MODIFY
+        @type evt: stc.EVT_STC_MODIFY (unused)
         @note: this method must complete its work very fast it gets
                called everytime a character is entered or removed from
                the document.
 
         """
-        pg_num = self.GetSelection()
-        title = self.GetPageText(pg_num)
-        if self.control.GetModify():
-            title = u"*" + title
-        if title != FNB.FlatNotebook.GetPageText(self, pg_num):
-            wx.CallAfter(self.SetPageText, pg_num, title)
-        
-    def UpdateTextControls(self):
+        try:
+            pg_num = self.GetSelection()
+            title = self.GetPageText(pg_num)
+            if self.control.GetModify():
+                title = u"*" + title
+            if title != FNB.FlatNotebook.GetPageText(self, pg_num):
+                self.SetPageText(pg_num, title)
+                ftitle = u"%s - file://%s" % (title, self.control.GetFileName())
+                self.frame.SetTitle(ftitle)
+        except wx.PyDeadObjectError:
+            pass
+
+    def UpdateTextControls(self, meth=None, args=list()):
         """Updates all text controls to use any new settings that have
         been changed since initialization.
         @postcondition: all stc controls in the notebook are reconfigured
@@ -590,8 +667,11 @@ class EdPages(FNB.FlatNotebook):
 
         """
         for control in self.GetTextControls():
-            control.UpdateAllStyles()
-            control.Configure()
+            if meth is not None:
+                getattr(control, meth)(*args)
+            else:
+                control.UpdateAllStyles()
+                control.Configure()
 
 #---- End Function Definitions ----#
 
@@ -608,13 +688,13 @@ def PromptToReSave(win, cfile):
                             _("%s has been deleted since its "
                               "last save point.\n\nWould you "
                               "like to save it again?") % cfile,
-                            _("Resave File?"), 
+                            _("Resave File?"),
                             wx.YES_NO | wx.ICON_INFORMATION)
     mdlg.CenterOnParent()
     result = mdlg.ShowModal()
     mdlg.Destroy()
     if result == wx.ID_YES:
-        win.control.SaveFile(cfile)
+        result = win.control.SaveFile(cfile)
     else:
         win.control.SetModTime(0)
 
@@ -624,10 +704,10 @@ def AskToReload(win, cfile):
     @param cfile: the file to prompt for a reload of
 
     """
-    mdlg = wx.MessageDialog(win.frame, 
+    mdlg = wx.MessageDialog(win.frame,
                             _("%s has been modified by another "
                               "application.\n\nWould you like "
-                              "to Reload it?") % cfile, 
+                              "to Reload it?") % cfile,
                               _("Reload File?"),
                               wx.YES_NO | wx.ICON_INFORMATION)
     mdlg.CenterOnParent()
@@ -636,10 +716,10 @@ def AskToReload(win, cfile):
     if result == wx.ID_YES:
         ret, rmsg = win.control.ReloadFile()
         if not ret:
-            mdlg = wx.MessageDialog(win.frame, 
-                                    _("Failed to reload %s:\n"
-                                      "Error: %s") % \
-                                      (cfile, rmsg),
+            errmap = dict(filename=cfile, errmsg=rmsg)
+            mdlg = wx.MessageDialog(win.frame,
+                                    _("Failed to reload %(filename)s:\n"
+                                      "Error: %(errmsg)s") % errmsg,
                                     _("Error"),
                                     wx.OK | wx.ICON_ERROR)
             mdlg.ShowModal()
