@@ -2,25 +2,20 @@
 # Name: ed_search.py                                                          #
 # Purpose: Text searching services and utilities                              #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# Copyright: (c) 2007,2008 Cody Precord <staff@editra.org>                    #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: ed_search                                                          #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-# SUMMARY:                                                                 #
-#     Provides various search controls and searching services for finding  #
-#  text in a document.                                                     #
-#                                                                          #
-#--------------------------------------------------------------------------#
+Provides various search controls and searching services for finding text in a
+document. The L{TextFinder} is a search service that can be used to search and
+highlight text in a StyledTextCtrl.
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_search.py 50146 2007-11-22 07:42:56Z CJP $"
-__revision__ = "$Revision: 50146 $"
+__svnid__ = "$Id: ed_search.py 54280 2008-06-19 02:00:20Z CJP $"
+__revision__ = "$Revision: 54280 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -43,7 +38,7 @@ class TextFinder(object):
     """
     def __init__(self, parent, getstc):
         """Initializes the Text Finding Service, the getstc argument
-        needs to be a function that returns reference to stc to perform 
+        needs to be a function that returns reference to stc to perform
         the search in.
         @param getstc: callable function that will return an stc
 
@@ -56,6 +51,30 @@ class TextFinder(object):
         self._data        = wx.FindReplaceData()
         self._data.SetFlags(wx.FR_DOWN)
 
+    def GetClientString(self, multiline=False):
+        """Get the selected text in the current client buffer. By default
+        it will only return the selected text if its on a single line.
+        @keyword multiline: Return text if it is multiple lines
+        @return: wx.stc.StyledTextCtrl
+
+        """
+        buffer =  self.FetchPool()
+        start, end = buffer.GetSelection()
+        rtext = buffer.GetSelectedText()
+        if start != end:
+            sline = buffer.LineFromPosition(start)
+            eline = buffer.LineFromPosition(end)
+            if not multiline and (sline != eline):
+                rtext = u''
+        return rtext
+
+    def GetClientSelection(self):
+        """Get the selected text from the client buffer if any
+        @return: string
+
+        """
+        return self.FetchPool().GetSelectedText()
+
     def GetData(self):
         """Return the FindReplace data
         @return: search data
@@ -70,7 +89,7 @@ class TextFinder(object):
 
         """
         # Map of search flags
-        flag_map = {  wx.FR_MATCHCASE : wx.stc.STC_FIND_MATCHCASE, 
+        flag_map = {  wx.FR_MATCHCASE : wx.stc.STC_FIND_MATCHCASE,
                       wx.FR_WHOLEWORD : wx.stc.STC_FIND_WHOLEWORD,
                       wx.FR_MATCHCASE | wx.FR_WHOLEWORD : \
                                         wx.stc.STC_FIND_MATCHCASE | \
@@ -113,6 +132,7 @@ class TextFinder(object):
                 # HACK to ensure selection is visible
                 sel = pool.GetSelection()
                 pool.SetSelection(sel[1], sel[0])
+                pool.EnsureVisible(pool.LineFromPosition(found))
             else:
                 # Try search from begining/end again
                 self.SetStart(pool)
@@ -136,7 +156,7 @@ class TextFinder(object):
                 # Couldnt find it anywhere so set screen back to start position
                 pool.ScrollToLine(self._posinfo['scroll'])
                 pool.SetCurrentPos(self._posinfo['start'])
-                pool.SetSelection(self._posinfo['start'], 
+                pool.SetSelection(self._posinfo['start'],
                                   self._posinfo['start'])
                 wx.Bell() # alert user to unfound string
             else:
@@ -155,24 +175,31 @@ class TextFinder(object):
             self.SetScroll(pool) # Save scroll pos
             pool.SetTargetStart(0)
             pool.SetTargetEnd(pool.GetLength())
-            pool.SetSearchFlags(flag_map[s_flags - wx.FR_DOWN] | \
+            pool.SetSearchFlags(flag_map[max(0, s_flags - wx.FR_DOWN)] | \
                                 wx.stc.STC_FIND_REGEXP)
             replaced = 0
-            while pool.SearchInTarget(query) > 0:
-                pool.SetSelection(pool.GetTargetStart(), pool.GetTargetEnd())
-                pool.ReplaceSelection(replacestring)
-                pool.SetTargetStart(pool.GetTargetEnd() - (len(query) - \
-                                    len(replacestring)))
-                pool.SetTargetEnd(pool.GetLength())
+            fail = 0
+            pool.BeginUndoAction()
+            while fail < 2: #pool.SearchInTarget(query) > 0:
+                # HACK if the search pattern is at the begining of the buffer
+                #      the search will fail the first time. So check twice
+                #      before giving up. ?Scintilla Bug?
+                if pool.SearchInTarget(query) < 0:
+                    fail += 1
+                    continue
+                pool.ReplaceTarget(replacestring)
                 replaced += 1
+                pool.SetTargetEnd(pool.GetLength())
+            pool.EndUndoAction()
+
             pool.ScrollToLine(self._posinfo['scroll'])
             pool.SetCurrentPos(self._posinfo['start']) # Move cursor to start
             pool.SetSelection(self._posinfo['start'], self._posinfo['start'])
-            dlg = wx.MessageDialog(self._parent, 
+            dlg = wx.MessageDialog(self._parent,
                                    _("Replace All Finished\n"
                                      "A Total of %d matches were replaced") % \
-                                     replaced, 
-                                    _("All Done"), 
+                                     replaced,
+                                    _("All Done"),
                                     wx.OK | wx.ICON_INFORMATION)
             dlg.CenterOnParent()
             dlg.ShowModal()
@@ -197,6 +224,16 @@ class TextFinder(object):
         """
         return self._data.GetFlags()
 
+    def GetStart(self):
+        """Get the value of the start position that was last set by a call
+        to L{SetStart}.
+        @return: integer position value
+        @note: usually the returned value is the cursors position from the
+               begining of the last search operation.
+
+        """
+        return self._poinfo['start']
+
     def OnFindClose(self, evt):
         """Destroy Find Dialog After Cancel is clicked in it
         @param evt: event that called this handler
@@ -217,6 +254,12 @@ class TextFinder(object):
             self._find_dlg.Destroy()
             self._find_dlg = None
 
+        # Check for a selection in the buffer and load that text if
+        # there is any and it is at most one line.
+        query = self.GetClientString()
+        if len(query):
+            self.SetQueryString(query)
+
         e_id = evt.GetId()
         if e_id == ed_glob.ID_FIND_REPLACE:
             self._find_dlg = wx.FindReplaceDialog(self._parent, self._data, \
@@ -228,8 +271,7 @@ class TextFinder(object):
         else:
             evt.Skip()
             return
-        if wx.Platform == '__WXMAC__' and Profile_Get('METAL', 'bool', False):
-            self._find_dlg.SetExtraStyle(wx.DIALOG_EX_METAL)
+
         self._find_dlg.CenterOnParent()
         try:
             self._find_dlg.Show()
@@ -282,13 +324,13 @@ class EdSearchCtrl(wx.SearchCtrl):
     """
     def __init__(self, parent, id_, value="", menulen=0, \
                  pos=wx.DefaultPosition, size=wx.DefaultSize, \
-                 style=wx.TE_RICH2):
+                 style=wx.TE_RICH2|wx.TE_PROCESS_ENTER):
         """Initializes the Search Control
         @param menulen: max length of history menu
 
         """
         wx.SearchCtrl.__init__(self, parent, id_, value, pos, size, style)
-        
+
         # Attributes
         self._parent     = parent
         # TEMP HACK
@@ -306,17 +348,24 @@ class EdSearchCtrl(wx.SearchCtrl):
         self.SetMenu(self.rmenu)
 
         # Bind Events
-        if wx.Platform == '__WXMSW__':
+        if wx.Platform in ['__WXMSW__', '__WXGTK__']:
             for child in self.GetChildren():
                 if isinstance(child, wx.TextCtrl):
                     child.Bind(wx.EVT_KEY_UP, self.ProcessEvent)
                     break
-
-        self.Bind(wx.EVT_KEY_UP, self.ProcessEvent)
+        else:
+            self.Bind(wx.EVT_KEY_UP, self.ProcessEvent)
         self.Bind(wx.EVT_SEARCHCTRL_CANCEL_BTN, self.OnCancel)
         self.Bind(wx.EVT_MENU, self.OnHistMenu)
 
     #---- Functions ----#
+    def AutoSetQuery(self, multiline=False):
+        """Autoload a selected string from the controls client buffer"""
+        query = self.FindService.GetClientString(multiline)
+        if len(query):
+            self.FindService.SetQueryString(query)
+            self.SetValue(query)
+
     def ClearSearchFlag(self, flag):
         """Clears a previously set search flag
         @param flag: flag to clear from search data
@@ -324,6 +373,39 @@ class EdSearchCtrl(wx.SearchCtrl):
         """
         self._flags ^= flag
 
+    def DoSearch(self, next=True):
+        """Do the search and move the selection
+        @keyword next: search next or previous
+
+        """
+        s_cmd = wx.wxEVT_COMMAND_FIND
+        if next:
+            self.SetSearchFlag(wx.FR_DOWN)
+        else:
+            if wx.FR_DOWN & self._flags:
+                self.ClearSearchFlag(wx.FR_DOWN)
+
+        if self.GetValue() == self._last:
+            s_cmd = wx.wxEVT_COMMAND_FIND_NEXT
+        self.InsertHistoryItem(self.GetValue())
+
+        self._last = self.GetValue()
+        self.FindService.SetQueryString(self.GetValue())
+        self.FindService.SetSearchFlags(self._flags)
+        self.FindService.OnFind(wx.FindDialogEvent(s_cmd))
+
+        # Give feedback on whether text was found or not
+        if self.FindService.GetLastFound() < 0 and len(self.GetValue()) > 0:
+            self.SetForegroundColour(wx.RED)
+            wx.Bell()
+        else:
+            # ?wxBUG? cant set text back to black after changing color
+            # But setting it to this almost black color works. Most likely its
+            # due to bit masking but I havent looked at the source so I am not
+            # sure
+            self.SetForegroundColour(wx.ColorRGB(0 | 1 | 0))
+        self.Refresh()
+
     def GetSearchData(self):
         """Gets the find data from the controls FindService
         @return: search data
@@ -372,7 +454,7 @@ class EdSearchCtrl(wx.SearchCtrl):
             try:
                 self.rmenu.RemoveItem(m_items[-1])
             except IndexError, msg:
-                wx.GetApp().GetLog()("[searchbar] menu error: %s" % str(msg))
+                wx.GetApp().GetLog()("[ed_search][err] menu error: %s" % str(msg))
 
     def IsMatchCase(self):
         """Returns True if the search control is set to search
@@ -444,7 +526,7 @@ class EdSearchCtrl(wx.SearchCtrl):
 
         e_key = evt.GetKeyCode()
         if e_key == wx.WXK_ESCAPE:
-            # HACK change to more safely determine the context
+            # TODO change to more safely determine the context
             # Currently control is only used in command bar
             self.GetParent().Hide()
             return
@@ -459,40 +541,18 @@ class EdSearchCtrl(wx.SearchCtrl):
 
         # Dont do search for navigation keys
         if tmp == wx.EmptyString or evt.CmdDown() or \
-           e_key in [wx.WXK_COMMAND, wx.WXK_LEFT, wx.WXK_RIGHT, 
+           e_key in [wx.WXK_COMMAND, wx.WXK_LEFT, wx.WXK_RIGHT,
                      wx.WXK_UP, wx.WXK_DOWN]:
             return
 
         s_cmd = wx.wxEVT_COMMAND_FIND
-        if e_key == wx.WXK_RETURN:
+        if e_key == wx.WXK_RETURN or e_key == wx.WXK_F3:
             if evt.ShiftDown():
-                if wx.FR_DOWN & self._flags:
-                    self.ClearSearchFlag(wx.FR_DOWN)
+                self.DoSearch(next=False)
             else:
-                self.SetSearchFlag(wx.FR_DOWN)
-
-            if self.GetValue() == self._last:
-                s_cmd = wx.wxEVT_COMMAND_FIND_NEXT
-            self.InsertHistoryItem(self.GetValue())
+                self.DoSearch(next=True)
         else:
-            self.SetSearchFlag(wx.FR_DOWN)
-
-        self._last = self.GetValue()
-        self.FindService.SetQueryString(self.GetValue())
-        self.FindService.SetSearchFlags(self._flags)
-        self.FindService.OnFind(wx.FindDialogEvent(s_cmd))
-
-        # Give feedback on whether text was found or not
-        if self.FindService.GetLastFound() < 0 and len(self.GetValue()) > 0:
-            self.SetForegroundColour(wx.RED)
-            wx.Bell()
-        else:
-            # ?wxBUG? cant set text back to black after changing color
-            # But setting it to this almost black color works. Most likely its
-            # due to bit masking but I havent looked at the source so I am not
-            # sure
-            self.SetForegroundColour(wx.ColorRGB(0 | 1 | 0))
-        self.Refresh()
+            self.DoSearch(next=True)
 
     def OnCancel(self, evt):
         """Cancels the Search Query
