@@ -3,28 +3,25 @@
 # Purpose: Editra's style management system. Implements the interpretation of #
 #          Editra Style Sheets to the StyledTextCtrl.                         #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: ed_style.py                                                        #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-# SUMMARY:                                                                 #
-# Provides a system for managing styles in the text control by parsing and #
-# managing data from Editra Style Sheets and translating it to be usable   #
-# with Editra's StyledTextCtrl. By default style sheets are only loaded    #
-# and parsed once per program execution to maximize performance and file   #
-# loading times.                                                           #
-#                                                                          #
-#--------------------------------------------------------------------------#
+Provides a system for managing styles in the text control. Compiles the data
+in an Editra Style Sheet to a format that Scintilla can understand. The
+specification of Editra Style Sheets that this module implements can be found
+either in the _docs_ folder of the source distribution or on Editra's home page
+U{http://editra.org/?page=docs&doc=ess_spec}.
+
+@summary: Style management system for managing the syntax highlighting of all
+          buffers
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: ed_style.py 50148 2007-11-22 07:45:48Z CJP $"
-__revision__ = "$Revision: 50148 $"
+__svnid__ = "$Id: ed_style.py 53301 2008-04-22 03:37:58Z CJP $"
+__revision__ = "$Revision: 53301 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
@@ -37,17 +34,23 @@ from profiler import Profile_Get, Profile_Set
 # Globals
 STY_ATTRIBUTES     = u"face fore back size"
 STY_EX_ATTRIBUTES  = u"eol bold italic underline"
+
+# Parser Values
+RE_ESS_COMMENT = re.compile("\/\*[^*]*\*+([^/][^*]*\*+)*\/")
+RE_ESS_SCALAR = re.compile("\%\([a-zA-Z0-9]+\)")
+RE_HEX_STR = re.compile("#[0-9a-fA-F]{3,6}")
+
 #--------------------------------------------------------------------------#
 
 class StyleItem(object):
     """A storage class for holding styling information
     @todo: The extra Attributes should be saved as a separate attribute in the
            StyleItem. This currenlty causes problems when customizing values in
-           the StyleEditor Changing this is fairly easy in this class but it 
+           the StyleEditor. Changing this is fairly easy in this class but it
            will require changes to the StyleMgr and Editor as well.
 
     """
-    def __init__(self, fore=wx.EmptyString, back=wx.EmptyString, 
+    def __init__(self, fore=wx.EmptyString, back=wx.EmptyString,
                        face=wx.EmptyString, size=wx.EmptyString):
         """Initiliazes the Style Object.
 
@@ -57,13 +60,13 @@ class StyleItem(object):
         @keyword size: Specifies font point size (int/formatted string)
 
         SPECIFICATION:
-          DATA FORMATS:
-            #123456       = hex color code
-            #123456,bold  = hex color code + extra style
-            Monaco        = Font Face Name
-            %(primary)s   = Format string to be swapped at runtime
-            10            = A font point size
-            %(size)s      = Format string to be swapped at runtime
+          - DATA FORMATS:
+            - #123456       = hex color code
+            - #123456,bold  = hex color code + extra style
+            - Monaco        = Font Face Name
+            - %(primary)s   = Format string to be swapped at runtime
+            - 10            = A font point size
+            - %(size)s      = Format string to be swapped at runtime
         """
         object.__init__(self)
         self.null = False
@@ -79,15 +82,12 @@ class StyleItem(object):
         @rtype: bool
 
         """
-        if str(self) == str(si2):
-            return True
-        else:
-            return False
+        return str(self) == str(si2)
 
     def __str__(self):
         """Converts StyleItem to a string
         @note: this return string is in a format that can be accepted by
-               Scintilla. Thus no spaces after the ':'
+               Scintilla. No spaces may be in the string after the ':'.
         @return: string representation of the StyleItem
 
         """
@@ -124,28 +124,28 @@ class StyleItem(object):
         @return: style items background attribute
 
         """
-        return self.back
+        return self.back.split(',')[0]
 
     def GetFace(self):
         """Returns the value of the face attribute
         @return: style items font face attribute
 
         """
-        return self.face
+        return self.face.split(',')[0]
 
     def GetFore(self):
         """Returns the value of the fore attribute
         @return: style items foreground attribute
 
         """
-        return self.fore
+        return self.fore.split(',')[0]
 
     def GetSize(self):
         """Returns the value of the size attribute as a string
         @return: style items font size attribute
 
         """
-        return self.size
+        return self.size.split(',')[0]
 
     def GetNamedAttr(self, attr):
         """Get the value of the named attribute
@@ -182,7 +182,7 @@ class StyleItem(object):
     #---- Set Functions ----#
     def SetAttrFromStr(self, style_str):
         """Takes style string and sets the objects attributes
-        by parsing the string for the values. Only sets or 
+        by parsing the string for the values. Only sets or
         overwrites values does not zero out previously set values.
         Returning True if value(s) are set or false otherwise.
         @param style_str: style information string (i.e fore:#888444)
@@ -269,7 +269,7 @@ class StyleItem(object):
             cur_str = cur_str.replace(u',' + ex_attr, wx.EmptyString)
             self.SetAttrFromStr(cur_str)
         else:
-            if ex_attr not in cur_str:
+            if u',' + ex_attr not in cur_str:
                 attr_map = { u"fore" : self.GetFore(),
                              u"back" : self.GetBack(),
                              u"face" : self.GetFace(),
@@ -284,14 +284,14 @@ class StyleItem(object):
 
     def SetNamedAttr(self, attr, value):
         """Sets a StyleItem attribute by named string.
-        @note: This is not intended to be used for setting extra 
+        @note: This is not intended to be used for setting extra
                attributes such as bold, eol, ect..
         @param attr: a particular attribute to set (i.e fore, face, back, size)
         @param value: value to set the attribute to contain
 
         """
         self.null = False
-        cur_val = getattr(self, attr)
+        cur_val = getattr(self, attr, None)
         if cur_val is not None:
             if u"," in cur_val:
                 tmp = cur_val.split(u",")
@@ -303,7 +303,7 @@ class StyleItem(object):
 
 class StyleMgr(object):
     """Manages style definitions and provides them on request.
-    Also provides functionality for loading custom style sheets and 
+    Also provides functionality for loading custom style sheets and
     modifying styles during run time.
 
     """
@@ -328,9 +328,9 @@ class StyleMgr(object):
 
         # Get the Style Set
         if custom != wx.EmptyString and self.LoadStyleSheet(custom):
-            self.LOG("[styles][init] Loaded custom style sheet %s" % custom)
+            self.LOG("[ed_style][info] Loaded custom style sheet %s" % custom)
         else:
-            self.LOG("[styles][err] Failed to import styles from %s" % custom)
+            self.LOG("[ed_style][err] Failed to import styles from %s" % custom)
 
     def BlankStyleDictionary(self):
         """Returns a dictionary of unset style items based on the
@@ -344,7 +344,7 @@ class StyleMgr(object):
             if key in ('select_style', 'whitespace_style'):
                 sty_dict[key] = NullStyleItem()
             else:
-                sty_dict[key] = StyleItem("#000000", "#FFFFFF", 
+                sty_dict[key] = StyleItem("#000000", "#FFFFFF",
                                           "%(primary)s", "%(size)d")
         return sty_dict
 
@@ -364,18 +364,18 @@ class StyleMgr(object):
         if font is not None:
             mfont = font
         else:
-            mfont = wx.Font(10, wx.FONTFAMILY_MODERN, wx.FONTSTYLE_NORMAL, 
+            mfont = wx.Font(10, wx.FONTFAMILY_MODERN, wx.FONTSTYLE_NORMAL,
                             wx.FONTWEIGHT_NORMAL)
             Profile_Set('FONT1', mfont, 'font')
         primary = mfont.GetFaceName()
 
         font = Profile_Get('FONT2', 'font', None)
         if font is None:
-            font = wx.Font(10, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL, 
+            font = wx.Font(10, wx.FONTFAMILY_SWISS, wx.FONTSTYLE_NORMAL,
                             wx.FONTWEIGHT_NORMAL)
             Profile_Set('FONT2', font, 'font')
         secondary = font.GetFaceName()
-        faces = { 
+        faces = {
                   self.FONT_PRIMARY   : primary,
                   self.FONT_SECONDARY : secondary,
                   self.FONT_SIZE  : mfont.GetPointSize(),
@@ -430,7 +430,7 @@ class StyleMgr(object):
         @keyword hex: return a hex string or colour object
         @type hex: bool
         @return: wx.Colour of default style background or hex value
-        @rtype: wx.Colour or string 
+        @rtype: wx.Colour or string
 
         """
         back = self.GetItemByName('default_style').GetBack()
@@ -450,13 +450,13 @@ class StyleMgr(object):
 
         """
         if self.HasNamedStyle(name):
-            if u"%" in unicode(self.STYLES[self.style_set][name]):
-                val = unicode(self.STYLES[self.style_set][name]) % self.fonts
+            if u"%" in unicode(StyleMgr.STYLES[self.style_set][name]):
+                val = unicode(StyleMgr.STYLES[self.style_set][name]) % self.fonts
                 item = StyleItem()
                 item.SetAttrFromStr(val)
                 return item
             else:
-                return self.STYLES[self.style_set][name]
+                return StyleMgr.STYLES[self.style_set][name]
         else:
             return StyleItem()
 
@@ -467,10 +467,10 @@ class StyleMgr(object):
 
         """
         if primary:
-            font = wx.FFont(self.fonts[self.FONT_SIZE], wx.DEFAULT, 
+            font = wx.FFont(self.fonts[self.FONT_SIZE], wx.DEFAULT,
                             face=self.fonts[self.FONT_PRIMARY])
         else:
-            font = wx.FFont(self.fonts[self.FONT_SIZE2], wx.DEFAULT, 
+            font = wx.FFont(self.fonts[self.FONT_SIZE2], wx.DEFAULT,
                             face=self.fonts[self.FONT_SECONDARY])
         return font
 
@@ -483,18 +483,18 @@ class StyleMgr(object):
 
         """
         if self.HasNamedStyle(name):
-            return str(self.GetItemByName(name))
+            return unicode(self.GetItemByName(name))
         else:
             return wx.EmptyString
 
     def GetStyleSet(self):
-        """Returns the current set of styles or the default set if 
+        """Returns the current set of styles or the default set if
         there is no current set.
         @return: current style set dictionary
         @rtype: dict
 
         """
-        return self.STYLES.get(self.style_set, DefaultStyleDictionary())
+        return StyleMgr.STYLES.get(self.style_set, DefaultStyleDictionary())
 
     def HasNamedStyle(self, name):
         """Checks if a style has been set/loaded or not
@@ -515,21 +515,21 @@ class StyleMgr(object):
         """
         if isinstance(style_sheet, basestring) and \
            os.path.exists(style_sheet) and \
-           ((force or not self.STYLES.has_key(style_sheet)) or \
+           ((force or not StyleMgr.STYLES.has_key(style_sheet)) or \
              style_sheet != self.style_set):
             reader = util.GetFileReader(style_sheet)
             if reader == -1:
-                self.LOG("[styles][err] Failed to open style sheet: %s" % style_sheet)
+                self.LOG("[ed_style][err] Failed to open style sheet: %s" % style_sheet)
                 return False
             ret_val = self.SetStyles(style_sheet, self.ParseStyleData(reader.read()))
             reader.close()
             return ret_val
-        elif not self.STYLES.has_key(style_sheet):
-            self.LOG("[styles] Style sheet %s does not exists" % style_sheet)
+        elif not StyleMgr.STYLES.has_key(style_sheet):
+            self.LOG("[ed_style][warn] Style sheet %s does not exists" % style_sheet)
             self.SetStyles('default', DefaultStyleDictionary())
             return False
         else:
-            self.LOG("[styles][info] Using cached style data")
+            self.LOG("[ed_style][info] Using cached style data")
             return True
 
     def PackStyleSet(self, style_set):
@@ -569,6 +569,9 @@ class StyleMgr(object):
                  data.
 
         """
+        # Remove all comments
+        style_data = RE_ESS_COMMENT.sub(u'', style_data)
+
         # Compact data into a contiguous string
         style_data = style_data.replace(u"\r\n", u"").replace(u"\n", u"")
         style_data = style_data.replace(u"\t", u"")
@@ -583,10 +586,10 @@ class StyleMgr(object):
         tmp = style_tree
         for style in tmp:
             if len(style) != 2:
-                self.LOG("[styles][error] There was an error parsing "
+                self.LOG("[ed_style][err] There was an error parsing "
                          "the syntax data from " + self.style_set)
-                self.LOG("[styles][error] You are missing a { or } " +
-                        "in Def: " + style[0].split()[0])
+                self.LOG("[ed_style][err] You are missing a { or } " +
+                         "in Def: " + style[0].split()[0])
                 if strict:
                     raise SyntaxError, \
                           "Missing { or } near Def: %s" % style[0].split()[0]
@@ -596,7 +599,7 @@ class StyleMgr(object):
         # Tree Level 2 Build small trees of tag and style attributes
         # Tree Level 3 Branch tree into TAG => Attr => Value String
         for branch in style_tree:
-            tmp2 = [leaf.strip().split(u":") 
+            tmp2 = [leaf.strip().split(u":")
                     for leaf in branch[1].strip().split(u";")]
             if len(tmp2) and tmp2[-1][0] == wx.EmptyString:
                 tmp2.pop()
@@ -609,14 +612,15 @@ class StyleMgr(object):
             value = list()
             tag = branch[0].replace(u" ", u"")
             for leaf in branch[1]:
-                leaf[0] = leaf[0].strip() # Remove any remaining whitespace
+                # Remove any remaining whitespace
+                leaf = [part.strip() for part in leaf]
                 if len(leaf) != 2:
-                    self.LOG("[styles][error] Missing a : or ; in the "
+                    self.LOG("[ed_style][err] Missing a : or ; in the "
                              "declaration of %s" % tag)
                     if strict:
                         raise SyntaxError, "Missing : or ; in def: %s" % tag
                 elif leaf[0] not in STY_ATTRIBUTES:
-                    self.LOG(("[styles][warning] Unknown style attribute: %s"
+                    self.LOG(("[ed_style][warn] Unknown style attribute: %s"
                              ", In declaration of %s") % (leaf[0], tag))
                     if strict:
                         raise SyntaxWarning, "Unknown attribute %s" % leaf[0]
@@ -633,17 +637,17 @@ class StyleMgr(object):
         # Validate leaf values and format into stylestring
         for style_def in style_dict:
             if not style_def[0][0].isalpha():
-                self.LOG("[styles][syntax_error] The style def %s is not a "
+                self.LOG("[ed_style][err] The style def %s is not a "
                          "valid name" % style_def[0])
                 if strict:
                     raise SyntaxError, "%s is an invalid name" % style_def[0]
             else:
                 style_str = wx.EmptyString
                 for attrib in style_dict[style_def]:
-                    values = [ val for val in attrib[1].split(u" ") 
+                    values = [ val for val in attrib[1].split(u" ")
                                if val != wx.EmptyString ]
                     if len(values) > 2:
-                        self.LOG("[styles][warning] Only one extra " +
+                        self.LOG("[ed_style][warn] Only one extra " +
                                  "attribute can be set per style. See " +
                                  style_def + " => " + attrib[0])
                         if strict:
@@ -651,25 +655,25 @@ class StyleMgr(object):
 
                     # Validate values
                     v1ok = v2ok = False
-                    if len(values) and attrib[0] in "fore back":
-                        if values[0][0] == u"#" and len(values[0]) == 7 and \
-                           values[0][1:].isalnum():
-                            v1ok = True
-                    elif len(values) and attrib[0] in "face size": 
-                        # TODO these regular expressions need work
-                        match1 = re.compile("\%\([a-zA-Z0-9]+\)")
-                        match2 = re.compile("[a-zA-Z0-9]+")
-                        if match1.match(values[0]) or match2.match(values[0]):
+                    if attrib[0] in "fore back" and RE_HEX_STR.match(values[0]):
+                        v1ok = True
+                    elif len(values) and attrib[0] == "size":
+                        if RE_ESS_SCALAR.match(values[0]) or values[0].isdigit():
                             v1ok = True
                         else:
-                            self.LOG("[styles] [syntax_warning] Bad value in %s"
+                            self.LOG("[ed_style][warn] Bad value in %s"
                                      " the value %s is invalid." % \
                                      (attrib[0], values[0]))
+                    elif len(values) and attrib[0] == "face":
+                        if len(values) == 2 and \
+                           values[1] not in STY_EX_ATTRIBUTES:
+                            values = [u' '.join(values)]
+                        v1ok = True
 
                     if len(values) == 2 and values[1] in STY_EX_ATTRIBUTES:
                         v2ok = True
                     elif len(values) == 2:
-                        self.LOG("[styles][warning] Unknown extra " + \
+                        self.LOG("[ed_style][warn] Unknown extra " + \
                                  "attribute '" + values[1] + \
                                  "' in attribute: " + attrib[0])
 
@@ -679,8 +683,8 @@ class StyleMgr(object):
                         value = values[0]
                     else:
                         continue
-                    
-                    style_str = u",".join([style_str, 
+
+                    style_str = u",".join([style_str,
                                            u":".join([attrib[0], value])])
 
                 if style_str != wx.EmptyString:
@@ -730,7 +734,7 @@ class StyleMgr(object):
         @param value: style item to set tag to
 
         """
-        self.STYLES[self.style_set][style_tag] = value
+        StyleMgr.STYLES[self.style_set][style_tag] = value
 
     def SetStyles(self, name, style_dict, nomerge=False):
         """Sets the managers style data and returns True on success.
@@ -743,7 +747,7 @@ class StyleMgr(object):
         """
         if nomerge:
             self.style_set = name
-            self.STYLES[name] = self.PackStyleSet(style_dict)
+            StyleMgr.STYLES[name] = self.PackStyleSet(style_dict)
             return True
 
         # Merge the given style set with the default set to fill in any
@@ -752,15 +756,24 @@ class StyleMgr(object):
             # Check for bad data
             for style in style_dict.values():
                 if not isinstance(style, StyleItem):
-                    self.LOG("[styles][error] Invalid data in style dictionary")
+                    self.LOG("[ed_style][err] Invalid data in style dictionary")
                     return False
 
             self.style_set = name
-            tmp = MergeStyles(DefaultStyleDictionary(), style_dict)
-            self.STYLES[name] = self.PackStyleSet(tmp)
+            # Set any undefined styles to match the default_style
+            for tag, item in DefaultStyleDictionary().iteritems():
+                if tag not in style_dict:
+                    if tag in ['select_style', 'whitespace_style']:
+                        style_dict[tag] = NullStyleItem()
+                    elif tag in ['foldmargin_style']:
+                        style_dict[tag] = style_dict['default_style']
+                    else:
+                        style_dict[tag] = item
+
+            StyleMgr.STYLES[name] = self.PackStyleSet(style_dict)
             return True
         else:
-            self.LOG("[styles][error] SetStyles expects a " \
+            self.LOG("[ed_style][err] SetStyles expects a " \
                      "dictionary of StyleItems")
             return False
 
@@ -768,7 +781,7 @@ class StyleMgr(object):
 # Utility Functions
 def DefaultStyleDictionary():
     """This is the default style values that are used for styling
-    documents. Its used as a fallback for undefined values in a 
+    documents. Its used as a fallback for undefined values in a
     style sheet.
     @note: incomplete style sheets are merged against this set to ensure
            a full set of definitions is avaiable
@@ -778,6 +791,7 @@ def DefaultStyleDictionary():
         {'brace_good' : StyleItem("#FFFFFF", "#0000FF,bold"),
          'brace_bad'  : StyleItem(back="#FF0000,bold"),
          'calltip'    : StyleItem("#404040", "#FFFFB8"),
+         'caret_line' : StyleItem(back="D8F8FF"),
          'ctrl_char'  : StyleItem(),
          'line_num'   : StyleItem(back="#C0C0C0", face="%(secondary)s", \
                                   size="%(size3)d"),
@@ -793,6 +807,7 @@ def DefaultStyleDictionary():
          'directive_style' : StyleItem("#0000FF,bold", face="%(secondary)s"),
          'dockey_style' : StyleItem("#0000FF"),
          'error_style' : StyleItem("#DD0101,bold", face="%(secondary)s"),
+         'foldmargin_style' : StyleItem(back="#D1D1D1"),
          'funct_style' : StyleItem("#008B8B,italic"),
          'global_style' : StyleItem("#007F7F,bold", face="%(secondary)s"),
          'guide_style' : StyleItem("#838383"),
@@ -803,11 +818,10 @@ def DefaultStyleDictionary():
          'keyword3_style' : StyleItem("#008B8B,bold"),
          'keyword4_style' : StyleItem("#9D2424"),
          'marker_style' : StyleItem("#FFFFFF", "#000000"),
-         'folder_style' : StyleItem("#FFFFFF", "#000000"),
          'number_style' : StyleItem("#DD0101"),
          'number2_style' : StyleItem("#DD0101,bold"),
          'operator_style' : StyleItem("#000000", face="%(primary)s,bold"),
-         'pre_style' : StyleItem("#AB39F2,bold"),               
+         'pre_style' : StyleItem("#AB39F2,bold"),
          'pre2_style' : StyleItem("#AB39F2,bold", "#FFFFFF"),
          'regex_style' : StyleItem("#008B8B"),
          'scalar_style' : StyleItem("#AB37F2,bold", face="%(secondary)s"),
