@@ -2,41 +2,38 @@
 # Name: style_editor.py                                                       #
 # Purpose: Syntax Highlighting configuration dialog                           #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE:	style_editor.py                                                    #
-# AUTHOR: Cody Precord                                                     #
-# LANGUAGE: Python                                                         #
-# SUMMARY:                                                                 #
-#    Provides an editor dialog for graphically editing how the text is     #
-# presented in the editor when syntax highlighting is turned on. It does   #
-# this by taking the data from the controls and formating it into an Editra#
-# Style Sheet that the editor can load to configure the styles of the text.#
-#                                                                          #
-#--------------------------------------------------------------------------#
+Provides an editor dialog for graphically editing how the text is presented in 
+the editor when syntax highlighting is turned on. It does this by taking the 
+data from the controls and formating it into an Editra Style Sheet that the 
+editor can load to configure the styles of the text.
+
+@summary: Gui for creating custom Editra Style Sheets
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: style_editor.py 50234 2007-11-25 04:59:18Z CJP $"
-__revision__ = "$Revision: 50234 $"
+__svnid__ = "$Id: style_editor.py 53723 2008-05-23 15:53:53Z CJP $"
+__revision__ = "$Revision: 53723 $"
 
 #--------------------------------------------------------------------------#
-# Dependancies
+# Imports
 import os
 import glob
 import wx
-import wx.lib.colourselect as  csel
+
+# Editra Imports
 import ed_glob
 from profiler import Profile_Get, Profile_Set
 import ed_stc
 from ed_style import StyleItem
-import ed_event
 import util
 import syntax.syntax as syntax
+import eclib.colorsetter as colorsetter
 
 # Function Aliases
 _ = wx.GetTranslation
@@ -71,14 +68,12 @@ class StyleEditor(wx.Dialog):
         """
         wx.Dialog.__init__(self, parent, id_, title, style=style)
 
-        if wx.Platform == '__WXMAC__' and Profile_Get('METAL', 'bool', False):
-            self.SetExtraStyle(wx.DIALOG_EX_METAL)
-
         # Attributes
         self.LOG = wx.GetApp().GetLog()
         self.preview = ed_stc.EditraStc(self, wx.ID_ANY, size=(-1, 200),
                                         style=wx.SUNKEN_BORDER, use_dt=False)
         self.styles_orig = self.preview.GetStyleSet()
+        self.preview.SetCaretLineVisible(True)
         self.styles_new = DuplicateStyleDict(self.styles_orig)
         self.preview.SetStyles('preview', self.styles_new, True)
         self.OpenPreviewFile('cpp')
@@ -105,9 +100,9 @@ class StyleEditor(wx.Dialog):
             self._settings = SettingsPanel(self)
 
         # Control Panel Left Column
-        left_colum.AddMany([((10, 10), 0), 
+        left_colum.AddMany([((10, 10), 0),
                             (self.__StyleSheets(), 0, wx.ALIGN_LEFT),
-                            ((10, 10), 0), 
+                            ((10, 10), 0),
                             (self.__LexerChoice(), 0, wx.ALIGN_LEFT),
                             ((10, 10), 0),
                             (self.__StyleTags(), 1, wx.ALIGN_LEFT|wx.EXPAND),
@@ -115,8 +110,8 @@ class StyleEditor(wx.Dialog):
         ctrl_sizer.Add(left_colum, 0, wx.ALIGN_LEFT)
 
         # Divider
-        ctrl_sizer.Add(wx.StaticLine(self.ctrl_pane, size=(-1, 2), 
-                                     style=wx.LI_VERTICAL), 
+        ctrl_sizer.Add(wx.StaticLine(self.ctrl_pane, size=(-1, 2),
+                                     style=wx.LI_VERTICAL),
                        0, wx.ALIGN_CENTER_HORIZONTAL | wx.EXPAND)
         ctrl_sizer.Add((5, 5), 0)
 
@@ -131,8 +126,8 @@ class StyleEditor(wx.Dialog):
 
         # Preview Area
         pre_sizer = wx.BoxSizer(wx.HORIZONTAL)
-        pre_sizer.AddMany([((10, 10), 0), 
-                           (wx.StaticText(self, label=_("Preview") + u": "), 
+        pre_sizer.AddMany([((10, 10), 0),
+                           (wx.StaticText(self, label=_("Preview") + u": "),
                             0, wx.ALIGN_LEFT)])
         sizer.AddMany([((10, 10), 0), (pre_sizer, 0, wx.ALIGN_LEFT),
                        (self.preview, 0, wx.EXPAND | wx.BOTTOM)])
@@ -141,7 +136,7 @@ class StyleEditor(wx.Dialog):
         b_sizer = wx.BoxSizer(wx.HORIZONTAL)
         ok_b = wx.Button(self, wx.ID_OK, _("Ok"))
         ok_b.SetDefault()
-        b_sizer.AddMany([(wx.Button(self, wx.ID_CANCEL, _("Cancel"))), 
+        b_sizer.AddMany([(wx.Button(self, wx.ID_CANCEL, _("Cancel"))),
                          (wx.Button(self, wx.ID_SAVE, _("Export"))), (ok_b)])
         sizer.Add(b_sizer, 0, wx.ALIGN_RIGHT |
                   wx.ALIGN_CENTER_VERTICAL | wx.ALL, 5)
@@ -160,7 +155,7 @@ class StyleEditor(wx.Dialog):
         self.Bind(wx.EVT_CHECKBOX, self.OnCheck)
         self.Bind(wx.EVT_CLOSE, self.OnClose)
         self.Bind(wx.EVT_LISTBOX, self.OnListBox)
-        self.Bind(ed_event.EVT_NOTIFY, self.OnColor)
+        self.Bind(colorsetter.EVT_COLORSETTER, self.OnColor)
         self.preview.Bind(wx.EVT_LEFT_UP, self.OnTextRegion)
         self.preview.Bind(wx.EVT_KEY_UP, self.OnTextRegion)
     #--- End Init ---#
@@ -173,9 +168,9 @@ class StyleEditor(wx.Dialog):
 
         """
         lex_sizer = wx.BoxSizer(wx.HORIZONTAL)
-        lexer_lbl = wx.StaticText(self.ctrl_pane, wx.ID_ANY, 
+        lexer_lbl = wx.StaticText(self.ctrl_pane, wx.ID_ANY,
                                   _("Syntax Files") + u": ")
-        lexer_lst = wx.Choice(self.ctrl_pane, ed_glob.ID_LEXER, 
+        lexer_lst = wx.Choice(self.ctrl_pane, ed_glob.ID_LEXER,
                               choices=syntax.GetLexerList())
         lexer_lst.SetToolTip(wx.ToolTip(_("Set the preview file type")))
         lexer_lst.SetStringSelection(u"CPP")
@@ -200,9 +195,9 @@ class StyleEditor(wx.Dialog):
         ss_choice.SetStringSelection(Profile_Get('SYNTHEME', 'str'))
         ss_new = wx.CheckBox(self.ctrl_pane, wx.ID_NEW, _("New"))
         ss_new.SetToolTip(wx.ToolTip(_("Start a blank new style")))
-        ss_sizer.AddMany([((10, 10)), (ss_lbl, 0, wx.ALIGN_CENTER_VERTICAL), 
+        ss_sizer.AddMany([((10, 10)), (ss_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
                           ((5, 0)),
-                          (ss_choice, 0, wx.ALIGN_CENTER_VERTICAL), ((10, 0)), 
+                          (ss_choice, 0, wx.ALIGN_CENTER_VERTICAL), ((10, 0)),
                           (ss_new, 0, wx.ALIGN_CENTER_VERTICAL), ((10, 10))])
         return ss_sizer
 
@@ -215,7 +210,7 @@ class StyleEditor(wx.Dialog):
         style_sizer = wx.BoxSizer(wx.HORIZONTAL)
         style_sizer2 = wx.BoxSizer(wx.VERTICAL)
 
-        style_lbl = wx.StaticText(self.ctrl_pane, wx.ID_ANY, 
+        style_lbl = wx.StaticText(self.ctrl_pane, wx.ID_ANY,
                                   _("Style Tags") + u": ")
 
         style_tags = self.styles_orig.keys()
@@ -227,9 +222,9 @@ class StyleEditor(wx.Dialog):
                                choices=sorted(style_tags), style=wx.LB_SINGLE)
         style_sizer2.AddMany([(style_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
                              (style_lst, 1, wx.EXPAND)])
-        style_sizer.AddMany([((10, 10), 0), 
-                             (style_sizer2, 1, 
-                              wx.ALIGN_CENTER_HORIZONTAL | wx.EXPAND), 
+        style_sizer.AddMany([((10, 10), 0),
+                             (style_sizer2, 1,
+                              wx.ALIGN_CENTER_HORIZONTAL | wx.EXPAND),
                              ((10, 10), 0)])
         return style_sizer
 
@@ -249,17 +244,17 @@ class StyleEditor(wx.Dialog):
 
         result = wx.ID_NO
         if diff:
-            dlg = wx.MessageDialog(self, 
+            dlg = wx.MessageDialog(self,
                                     _("Some styles have been changed would "
                                       "you like to save before exiting?"),
-                                   _("Save Styles"), 
+                                   _("Save Styles"),
                                    style=wx.YES_NO | wx.YES_DEFAULT | \
                                          wx.CANCEL | wx.ICON_INFORMATION)
             dlg.CenterOnParent()
             result = dlg.ShowModal()
             dlg.Destroy()
         return result
-        
+
     def EnableSettings(self, enable=True):
         """Enables/Disables all settings controls
         @keyword enable: whether to enable/disable settings controls
@@ -274,7 +269,7 @@ class StyleEditor(wx.Dialog):
 
         """
         if ed_glob.CONFIG['STYLES_DIR'] == ed_glob.CONFIG['SYS_STYLES_DIR']:
-            user_config = os.path.join(wx.GetHomeDir(), 
+            user_config = os.path.join(wx.GetHomeDir(),
                                         "." + ed_glob.PROG_NAME, 'styles')
             if not os.path.exists(user_config):
                 try:
@@ -315,11 +310,7 @@ class StyleEditor(wx.Dialog):
                 if sheet_path.startswith(ed_glob.CONFIG['STYLES_DIR']) or \
                    sheet_path.startswith(ed_glob.CONFIG['SYS_STYLES_DIR']):
                     # Update editor windows/buffer to use new style sheet
-                    Profile_Set('SYNTHEME', sheet)
-                    for mainw in wx.GetApp().GetMainWindows():
-                        mainw.nb.UpdateTextControls()
-                        mainw.SetStatusText(_("Changed color scheme to %s") % \
-                                            sheet, ed_glob.SB_INFO)
+                    UpdateBufferStyles(sheet)
         dlg.Destroy()
         return result
 
@@ -331,7 +322,7 @@ class StyleEditor(wx.Dialog):
         """
         sty_sheet = list()
         ditem = self.styles_new.get('default_style', StyleItem())
-        dvals = ';\n\t\t'.join([item.replace(',', ' ') 
+        dvals = ';\n\t\t'.join([item.replace(',', ' ')
                                 for item in ditem.GetAsList() ]) + ';'
         sty_sheet.append(''.join(['default_style {\n\t\t', dvals, '\n\n}\n\n']))
         for tag, item in self.styles_new.iteritems():
@@ -344,7 +335,7 @@ class StyleEditor(wx.Dialog):
                 if ival is None or ival == ditem.GetNamedAttr(attr):
                     continue
 
-                stage1 = ''.join((stage1, attr, u':', 
+                stage1 = ''.join((stage1, attr, u':',
                                   ival.replace(',', ' '), u';'))
 
             if len(stage1):
@@ -361,7 +352,7 @@ class StyleEditor(wx.Dialog):
         @param evt: event that called this handler
 
         """
-        self.LOG('[style_editor][cancel] Cancel Clicked Closing Window')
+        self.LOG('[style_editor][evt] Cancel Clicked Closing Window')
         evt.Skip()
 
     def OnCheck(self, evt):
@@ -439,8 +430,6 @@ class StyleEditor(wx.Dialog):
         selection in the style tags list to the style tag of the area
         the cursor has moved into.
         @param evt: event that called this handler
-        @todo: optimizations, find out why UpdateBaseStyles is called 8 times
-               everytime this is fired.
 
         """
         if evt is not None:
@@ -475,9 +464,14 @@ class StyleEditor(wx.Dialog):
         @param evt: event that called this handler
 
         """
-        self.LOG('[style_editor][info] Ok Clicked Closing Window')
+        self.LOG('[style_editor][evt] Ok Clicked Closing Window')
+
+
         result = self.DiffStyles()
         if result == wx.ID_NO:
+            # Get Current Selection to update buffers
+            csheet = self.FindWindowById(ed_glob.ID_PREF_SYNTHEME).GetStringSelection()
+            UpdateBufferStyles(csheet)
             evt.Skip()
         elif result == wx.ID_CANCEL:
             self.LOG('[style_editor][info] canceled closing')
@@ -491,7 +485,7 @@ class StyleEditor(wx.Dialog):
         @param evt: event that called this handler
 
         """
-        self.LOG('[style_editor][export] Saving style changes')
+        self.LOG('[style_editor][evt] Export Clicked')
         self.ExportStyleSheet()
 
     def OpenPreviewFile(self, file_lbl):
@@ -501,6 +495,7 @@ class StyleEditor(wx.Dialog):
 
         """
         fname = file_lbl.replace(u" ", u"_").replace(u"/", u"_").lower()
+        fname = fname.replace('#', 'sharp')
         try:
             fname = glob.glob(ed_glob.CONFIG['TEST_DIR'] + fname + ".*")[0]
         except IndexError:
@@ -520,7 +515,7 @@ class StyleEditor(wx.Dialog):
         @param syntax_data: syntax data set to configure panel from
 
         """
-        val_str = str(syntax_data)
+        val_str = unicode(syntax_data)
         val_map = { ID_FORE_COLOR : syntax_data.GetFore(),
                     ID_BACK_COLOR : syntax_data.GetBack(),
                     ID_BOLD       : "bold" in val_str,
@@ -545,10 +540,10 @@ class StyleEditor(wx.Dialog):
                 ctrl.SetValue(val_map[sid])
             elif c_type == "wxChoice":
                 ctrl.SetStringSelection(val_map[sid])
-            elif isinstance(ctrl, ColourSetter):
+            elif isinstance(ctrl, colorsetter.ColorSetter):
                 ctrl.SetLabel(val_map[sid][:7])
-                ctrl.SetValue(wx.Color(int(val_map[sid][1:3], 16), 
-                                       int(val_map[sid][3:5], 16), 
+                ctrl.SetValue(wx.Color(int(val_map[sid][1:3], 16),
+                                       int(val_map[sid][3:5], 16),
                                        int(val_map[sid][5:7], 16)))
         return True
 
@@ -571,7 +566,7 @@ class StyleEditor(wx.Dialog):
             val = ctrl.GetValue()
         elif ctrl_t == 'wxChoice':
             val = ctrl.GetStringSelection()
-        elif isinstance(ctrl, ColourSetter):
+        elif isinstance(ctrl, colorsetter.ColorSetter):
             val = ctrl.GetLabel()
         else:
             return False
@@ -610,7 +605,7 @@ class SettingsPanel(wx.Panel):
         wx.Panel.__init__(self, parent)
 
         # Attributes
-        
+
         # Layout
         self.__DoLayout()
 
@@ -621,17 +616,17 @@ class SettingsPanel(wx.Panel):
 
         # Settings top
         setting_sizer.Add((10, 10))
-        cbox_sizer = wx.StaticBoxSizer(wx.StaticBox(self, 
+        cbox_sizer = wx.StaticBoxSizer(wx.StaticBox(self,
                                           label=_("Color") + u":"), wx.VERTICAL)
 
         # Foreground
         fground_sizer = wx.BoxSizer(wx.HORIZONTAL)
         fground_lbl = wx.StaticText(self, label=_("Foreground") + u": ")
-        fground_sel = ColourSetter(self, ID_FORE_COLOR, "#000000")
-        fground_sizer.AddMany([((5, 5)), 
+        fground_sel = colorsetter.ColorSetter(self, ID_FORE_COLOR, wx.BLACK)
+        fground_sizer.AddMany([((5, 5)),
                                (fground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
                                ((2, 2), 1, wx.EXPAND),
-                               (fground_sel, 0, wx.ALIGN_CENTER_VERTICAL), 
+                               (fground_sel, 0, wx.ALIGN_CENTER_VERTICAL),
                                ((5, 5))])
         cbox_sizer.AddMany([(fground_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND),
                             ((10, 10))])
@@ -639,11 +634,11 @@ class SettingsPanel(wx.Panel):
         # Background
         bground_sizer = wx.BoxSizer(wx.HORIZONTAL)
         bground_lbl = wx.StaticText(self, label=_("Background") + u": ")
-        bground_sel = ColourSetter(self, ID_BACK_COLOR, "#FFFFFF")
-        bground_sizer.AddMany([((5, 5)), 
+        bground_sel = colorsetter.ColorSetter(self, ID_BACK_COLOR, wx.WHITE)
+        bground_sizer.AddMany([((5, 5)),
                                (bground_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
                                ((2, 2), 1, wx.EXPAND),
-                               (bground_sel, 0, wx.ALIGN_CENTER_VERTICAL), 
+                               (bground_sel, 0, wx.ALIGN_CENTER_VERTICAL),
                                ((5, 5))])
         cbox_sizer.Add(bground_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND)
         setting_top.AddMany([(cbox_sizer, 0, wx.ALIGN_TOP), ((10, 10))])
@@ -686,11 +681,11 @@ class SettingsPanel(wx.Panel):
         fsizes = ['%(size)d', '%(size2)d']
         fsizes.extend([ str(x) for x in xrange(4, 21) ])
         fs_choice = wx.Choice(self, ID_FONT_SIZE, choices=fsizes)
-        fsize_sizer.AddMany([((5, 5), 0), 
+        fsize_sizer.AddMany([((5, 5), 0),
                              (fsize_lbl, 0, wx.ALIGN_CENTER_VERTICAL),
-                             (fs_choice, 1, wx.EXPAND | wx.ALIGN_RIGHT), 
+                             (fs_choice, 1, wx.EXPAND | wx.ALIGN_RIGHT),
                              ((5, 5), 0)])
-        fbox_sizer.AddMany([((5, 5)), 
+        fbox_sizer.AddMany([((5, 5)),
                             (fsize_sizer, 0, wx.ALIGN_LEFT | wx.EXPAND)])
         fh_sizer.AddMany([(fbox_sizer, 0, wx.ALIGN_CENTER_HORIZONTAL),
                           ((10, 10))])
@@ -702,159 +697,12 @@ class SettingsPanel(wx.Panel):
         self.SetSizer(setting_sizer)
         self.SetAutoLayout(True)
 
-#-----------------------------------------------------------------------------#
-class ColourSetter(wx.Panel):
-    """Control for setting and selecting a color to describe the
-    various styling of the text control.
-    
-    """
-    def __init__(self, parent, id_, label=wx.EmptyString):
-        """Create the control, it is a composite of a colourSelect and
-        and a text control.
-        @keyword label: the hex string value to go in the text portion
-
-        """
-        wx.Panel.__init__(self, parent, id_)
-
-        # Attributes
-        self._label = label
-        self._txt = wx.TextCtrl(self, value=label, style=wx.TE_CENTER)
-        txtheight = self._txt.GetTextExtent('#000000')[1]
-        self._txt.SetMaxSize((-1, txtheight + 4))
-        self._txt.SetToolTip(wx.ToolTip(_("Enter a hex color value")))
-        self._cbtn = csel.ColourSelect(self, colour=util.HexToRGB(label), 
-                                       size=(20, 20))
-
-        self._DoLayout()
-
-        # Event Handlers
-        self.Bind(csel.EVT_COLOURSELECT, self.OnColour)
-        self._txt.Bind(wx.EVT_KEY_UP, self.OnTextChange)
-        self._txt.Bind(wx.EVT_TEXT_PASTE, self.OnTextChange)
-        self._txt.Bind(wx.EVT_KEY_DOWN, self.OnValidateTxt)
-        self.Bind(wx.EVT_UPDATE_UI, self.OnUpdateUI)
-
-    def __PostEvent(self):
-        """Notify the parent window of any value changes to the control"""
-        evt = ed_event.NotificationEvent(ed_event.edEVT_NOTIFY, self.GetId())
-        wx.PostEvent(self.GetParent(), evt)
-
-    def _DoLayout(self):
-        """Layout the controls"""
-        sizer = wx.BoxSizer(wx.HORIZONTAL)
-        sizer.Add(self._txt, 0, wx.EXPAND)
-        sizer.Add((5, 5), 0)
-        sizer.Add(self._cbtn, 0, wx.ALIGN_LEFT)
-        self.SetSizer(sizer)
-
-    def GetColour(self):
-        """Returns the colour value of the control
-        @return: wxColour object
-
-        """
-        return self._cbtn.GetValue()
-
-    def GetLabel(self):
-        """Gets the hex value from the text control
-        @return: string '#123456'
-        @note: ensures a full 6 digit hex value is returned, padding
-               with zero's where necessary
-
-        """
-        hexstr = self._txt.GetValue()
-        hexstr = hexstr.replace('#', '').replace(' ', '')
-        hexstr = '#' + hexstr + ('0' * (6 - len(hexstr)))
-        return hexstr
-
-    def OnColour(self, evt):
-        """Update the button and text control value
-        when a choice is made in the colour dialog.
-        @param evt: EVT_COLOURSELECT
-
-        """
-        e_val = evt.GetValue()[0:3]
-        red = hex(e_val[0])
-        green = hex(e_val[1])
-        blue = hex(e_val[2])
-        hex_str = "#%s%s%s" % (red[2:].zfill(2).upper(),
-                               green[2:].zfill(2).upper(),
-                               blue[2:].zfill(2).upper())
-        self._txt.SetValue(hex_str)
-        self._cbtn.SetValue(wx.Color(e_val[0], e_val[1], e_val[2]))
-        self.__PostEvent()
-
-    def OnTextChange(self, evt=None):
-        """Catch when text changes in the text control and update
-        button accordingly.
-        @keyword evt: event that called this handler
-
-        """
-        hexstr = self._txt.GetValue().replace('#', '')
-        if len(hexstr) > 6:
-            self._txt.SetValue(u'#' + hexstr[:6])
-        hexstr = '#' + hexstr + (u'0' * (6 - len(hexstr)))
-        self._cbtn.SetValue(util.HexToRGB(hexstr))
-        self.__PostEvent()
-
-    def OnUpdateUI(self, evt):
-        """Cleanup any bad characters and formating in the display
-        @param evt: UpdateUI
-
-        """
-        ret = u''
-        for char in self._txt.GetValue().replace('#', ''):
-            if char in '0123456789abcdefABCDEF':
-                ret = ret + char
-        self._txt.SetValue(u'#' + ret)
-
-    def OnValidateTxt(self, evt):
-        """Validate text to ensure only valid hex characters are entered
-        @param evt: wxEVT_KEY_DOWN
-
-        """
-        code = evt.GetKeyCode()
-        if code in [wx.WXK_DELETE, wx.WXK_BACK, wx.WXK_LEFT, wx.WXK_RIGHT] or \
-           evt.CmdDown():
-            evt.Skip()
-            return
-
-        key = unichr(code)
-        if (key.isdigit() and evt.ShiftDown()) or \
-           evt.AltDown() or evt.MetaDown():
-            return
-
-        if key in "0123456789ABCDEFabcdef#" and \
-           (len(self._txt.GetValue().lstrip("#")) < 6 or \
-            self._txt.GetStringSelection()):
-            evt.Skip()
-
-    def SetLabel(self, label):
-        """Set the label value of the text control
-        @param label: hex string to set label to
-
-        """
-        self._txt.SetValue(label)
-        self.OnTextChange()
-
-    def SetValue(self, colour):
-        """Set the color value of the button
-        @param colour: wxColour or 3 tuple to set colour value to
-
-        """
-        self._cbtn.SetValue(colour)
-        red, green, blue = colour[0:3]
-        hex_str = "#%s%s%s" % (hex(red)[2:].zfill(2).upper(),
-                               hex(green)[2:].zfill(2).upper(),
-                               hex(blue)[2:].zfill(2).upper())
-        self._txt.SetValue(hex_str)
-        self.__PostEvent()
-
 #-----------------------------------------------------------------------------#
 # Utility funtcions
 def DuplicateStyleDict(style_dict):
     """Duplicates the style dictionary to make a true copy of
     it, as simply assigning the dictionary to two different variables
-    only copies a reference leaving both variables pointing to the 
+    only copies a reference leaving both variables pointing to the
     same object.
     @param style_dict: dictionary of tags->StyleItems
     @return: a copy of the given styleitem dictionary
@@ -863,7 +711,22 @@ def DuplicateStyleDict(style_dict):
     new_dict = dict()
     for tag in style_dict:
         new_dict[tag] = StyleItem()
-        ok = new_dict[tag].SetAttrFromStr(unicode(style_dict[tag]))
-        if not ok:
+        is_ok = new_dict[tag].SetAttrFromStr(unicode(style_dict[tag]))
+        if not is_ok:
             new_dict[tag].null = True
     return new_dict
+
+def UpdateBufferStyles(sheet):
+    """Update the style used in all buffers
+    @sheet: Style sheet to use
+
+    """
+    # Only update if the sheet has changed
+    if sheet is None or sheet == Profile_Get('SYNTHEME'):
+        return
+
+    Profile_Set('SYNTHEME', sheet)
+    for mainw in wx.GetApp().GetMainWindows():
+        mainw.nb.UpdateTextControls('UpdateAllStyles')
+        mainw.SetStatusText(_("Changed color scheme to %s") % \
+                            sheet, ed_glob.SB_INFO)
