@@ -2,43 +2,32 @@
 # Name: util.py                                                               #
 # Purpose: Misc utility functions used through out Editra                     #
 # Author: Cody Precord <cprecord@editra.org>                                  #
-# Copyright: (c) 2007 Cody Precord <staff@editra.org>                         #
-# Licence: wxWindows Licence                                                  #
+# Copyright: (c) 2008 Cody Precord <staff@editra.org>                         #
+# License: wxWindows License                                                  #
 ###############################################################################
 
 """
-#--------------------------------------------------------------------------#
-# FILE: util.py                                                            #
-# LANGUAGE: Python                                                         #
-#                                                                          #
-# SUMMARY:                                                                 #
-# This file contains various helper functions and utilities that the       #
-# program uses. Basically a random library of misfit functions.	           #
-#                                                                          #
-# METHODS:                                                                 #
-# - FileDropTarget: Is a class that handles drag and drop events for the   #
-#                   the interface.                                         #
-# - GetPathChar: Returns the character used in building paths '\\' for     #
-#                windows and '/' for linux and mac.                        #
-# - GetFileName: Returns the name of the file from a given string          #
-# - GetExtension: Returns the extension of a file.                         #
-#--------------------------------------------------------------------------#
+This file contains various helper functions and utilities that the program uses.
+
 """
 
 __author__ = "Cody Precord <cprecord@editra.org>"
-__svnid__ = "$Id: util.py 50305 2007-11-28 08:15:04Z CJP $"
-__revision__ = "$Revision: 50305 $"
+__svnid__ = "$Id: util.py 54270 2008-06-17 03:01:48Z CJP $"
+__revision__ = "$Revision: 54270 $"
 
 #--------------------------------------------------------------------------#
 # Dependancies
 import os
 import sys
+import stat
 import codecs
-import mimetypes
+import urllib2
 import wx
-import ed_event
+
+# Editra Libraries
 import ed_glob
-from syntax.syntax import GetFileExtensions
+import ed_event
+import ed_crypt
 import dev_tool
 
 _ = wx.GetTranslation
@@ -57,7 +46,7 @@ class DropTargetFT(wx.PyDropTarget):
         """
         wx.PyDropTarget.__init__(self)
         self.window = window
-        self._data = dict(data=None, fdata=None, tdata=None, 
+        self._data = dict(data=None, fdata=None, tdata=None,
                           tcallb=textcallback, fcallb=filecallback)
         self._tmp = None
         self._lastp = None
@@ -81,7 +70,7 @@ class DropTargetFT(wx.PyDropTarget):
                 longest = ext
 
         cords = [ (0, x * longest[1]) for x in xrange(len(txt)) ]
-        mdc = wx.MemoryDC(wx.EmptyBitmap(longest[0] + 5, 
+        mdc = wx.MemoryDC(wx.EmptyBitmap(longest[0] + 5,
                                          longest[1] * len(txt), 32))
         mdc.SetBackgroundMode(wx.TRANSPARENT)
         mdc.SetTextForeground(stc.GetDefaultForeColour())
@@ -106,16 +95,20 @@ class DropTargetFT(wx.PyDropTarget):
         @return: result of drop object entering window
 
         """
-        try:
-            if self.GetData():
-                files = self._data['fdata'].GetFilenames()
-                text = self._data['tdata'].GetText()
-            else:
-                return drag_result
-        except wx.PyAssertionError:
-            return wx.DragError
+        # GetData seems to happen automatically on msw, calling it again
+        # causes this to fail the first time.
+        if wx.Platform in ['__WXGTK__', '__WXMSW__']:
+            return wx.DragCopy
+
+        if wx.Platform == '__WXMAC__':
+            try:
+                self.GetData()
+            except wx.PyAssertionError:
+                return wx.DragError
 
         self._lastp = (x_cord, y_cord)
+        files = self._data['fdata'].GetFilenames()
+        text = self._data['tdata'].GetText()
         if len(files):
             self.window.SetCursor(wx.StockCursor(wx.CURSOR_COPY_ARROW))
         else:
@@ -163,6 +156,9 @@ class DropTargetFT(wx.PyDropTarget):
 
         """
         self.window.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
+        if self.window.HasCapture():
+            self.window.ReleaseMouse()
+
         try:
             data = self.GetData()
         except wx.PyAssertionError:
@@ -179,14 +175,15 @@ class DropTargetFT(wx.PyDropTarget):
             if len(files) > 0 and self._data['fcallb'] is not None:
                 self._data['fcallb'](files)
             elif(len(text) > 0):
-                if SetClipboardText(text):
-                    win = self.window
-                    pos = win.PositionFromPointClose(x_cord, y_cord)
-                    if pos != wx.stc.STC_INVALID_POSITION:
-                        win.SetSelection(pos, pos)
-                        win.Paste()
-                    else:
-                        drag_result = wx.DragCancel
+                if hasattr(self.window, 'PositionFromPointClose'):
+                    pos = self.window.PositionFromPointClose(x_cord, y_cord)
+                else:
+                    pos = wx.stc.STC_INVALID_POSITION
+
+                if pos != wx.stc.STC_INVALID_POSITION:
+                    self.window.InsertText(pos, text)
+                else:
+                    drag_result = wx.DragCancel
         self.InitObjects()
         return drag_result
 
@@ -196,135 +193,92 @@ class DropTargetFT(wx.PyDropTarget):
 
         """
         self.window.SetCursor(wx.StockCursor(wx.CURSOR_ARROW))
+        if self.window.HasCapture():
+            self.window.ReleaseMouse()
+
         if self._tmp is not None:
-            self._tmp.EndDrag()
+            try:
+                self._tmp.EndDrag()
+            except wx.PyAssertionError, msg:
+                Log("[droptargetft][err] %s" % str(msg))
 
 #---- End FileDropTarget ----#
 
 #---- Misc Common Function Library ----#
-def SetClipboardText(txt):
-    """Copies text to the clipboard
-    @param txt: text to put in clipboard
+# Used for holding the primary selection on mac/msw
+FAKE_CLIPBOARD = None
+
+def GetClipboardText(primary=False):
+    """Get the primary selection from the clipboard if there is one
+    @return: str or None
 
     """
+    if primary and wx.Platform == '__WXGTK__':
+        wx.TheClipboard.UsePrimarySelection(True)
+    elif primary:
+        # Fake the primary selection on mac/msw
+        global FAKE_CLIPBOARD
+        return FAKE_CLIPBOARD
+    else:
+        pass
+
+    text_obj = wx.TextDataObject()
+    rtxt = None
+    if wx.TheClipboard.Open():
+        if wx.TheClipboard.GetData(text_obj):
+            rtxt = text_obj.GetText()
+        wx.TheClipboard.Close()
+
+    if primary and wx.Platform == '__WXGTK__':
+        wx.TheClipboard.UsePrimarySelection(False)
+    return rtxt
+
+def SetClipboardText(txt, primary=False):
+    """Copies text to the clipboard
+    @param txt: text to put in clipboard
+    @keyword primary: Set txt as primary selection (x11)
+
+    """
+    # Check if using primary selection
+    if primary and wx.Platform == '__WXGTK__':
+        wx.TheClipboard.UsePrimarySelection(True)
+    elif primary:
+        # Fake the primary selection on mac/msw
+        global FAKE_CLIPBOARD
+        FAKE_CLIPBOARD = txt
+        return True
+    else:
+        pass
+
     data_o = wx.TextDataObject()
     data_o.SetText(txt)
     if wx.TheClipboard.Open():
         wx.TheClipboard.SetData(data_o)
         wx.TheClipboard.Close()
-        return 1
-    return 0
-
-# File Helper Functions
-BOM = { 'utf-8' : codecs.BOM_UTF8,
-        'utf-16-be' : codecs.BOM_UTF16_BE,
-        'utf-16-le' : codecs.BOM_UTF16_LE,
-        'utf-7' : '+\v8-',
-        'latin-1' : '',
-        'ascii' : '' }
-
-# When no BOM is present this determines decode test order
-ENC = [ 'utf-8', 'utf-7', 'latin-1', 'utf-16-be', 'utf-16-le', 'ascii']
-  #      'utf-32-be', 'utf-32-le', 
-
-def DecodeString(str2decode):
-    """Decode a given string if possible and return that string
-    @param str2decode: the string to decode
-
-    """
-    decoded = str2decode
-    for enc in ENC:
-        try:
-            decoded = str2decode.decode(enc)
-        except (UnicodeDecodeError, UnicodeWarning):
-            continue
-        else:
-            break
-
-    return decoded
-
-def GetDecodedText(fname):
-    """Gets the text from a file and decodes the text using
-    a compatible decoder. Returns a tuple of the text and the
-    encoding it was decoded from.
-    @param fname: name of file to open and get text from
-    @return: tuple of (text, encoding string)
-    @note: must allow exceptions to be raised (side effect)
-
-    """
-    f_handle = file(fname, 'rb')
-    txt = f_handle.read()
-    f_handle.close()
-
-    # First try looking for a bom byte
-    tenc = ENC
-    for enc, bom in BOM.iteritems():
-        if txt.startswith(bom) and enc not in ['ascii', 'latin-1']:
-            if enc in tenc:
-                tenc.remove(enc)
-            tenc.insert(0, enc)
-            break
-
-    decoded = None
-    for enc in tenc:
-        try:
-            decoded = txt.decode(enc)
-        except (UnicodeDecodeError, UnicodeWarning):
-            continue
-        else:
-            break
-
-    if 'enc' not in locals():
-        enc = u''
-
-    if decoded:
-        dev_tool.DEBUGP("[txtdecoder] Decoded text as %s" % enc)
-        return decoded, enc
+        if primary and wx.Platform == '__WXGTK__':
+            wx.TheClipboard.UsePrimarySelection(False)
+        return True
     else:
-        dev_tool.DEBUGP("[txtdecoder][err] Decode Failed")
-        return txt, enc
+        return False
 
 def FilterFiles(file_list):
     """Filters a list of paths and returns a list of paths
-    that are valid, not directories, and not seemingly not binary.
+    that can probably be opened in the editor.
     @param file_list: list of files/folders to filter for good files in
-    @todo: find a better way to check for files that can be opened
 
     """
     good = list()
     for path in file_list:
-        if not os.path.exists(path) or os.path.isdir(path):
+        # Filter out directories and some common filetypes that can't
+        # be opened.
+        if not os.path.exists(path) or os.path.isdir(path) or \
+           GetExtension(path).lower() in ['gz', 'tar', 'bz2', 'zip', 'rar',
+                                          'ace', 'png', 'jpg', 'gif', 'jpeg',
+                                          'bmp', 'exe', 'pyc', 'pyo', 'psd',
+                                          'a', 'o', 'dll', 'ico', 'icns']:
             continue
         else:
-            # Check for binary files
-            # 1. Keep all files types we know about and all that have a mime
-            #    type of text.
-            mime = mimetypes.guess_type(path)
-            if GetExtension(path) in GetFileExtensions() or \
-               (mime[0] and u'text' in mime[0]):
-                good.append(path)
-            # 2. Throw out common filetypes we cant open...HACK
-            elif GetExtension(path).lower() in ['gz', 'tar', 'bz2', 'zip',
-                                                'rar', 'ace', 'png', 'jpg', 
-                                                'gif', 'jpeg', 'exe', 'pyc',
-                                                'pyo', 'psd']:
-                continue
-            # 3. Try to judge if we can open the file or not by sampling
-            #    some of the data, if 10% of the data is bad, drop the file.
-            else:
-                try:
-                    fhandle = file(path, "rb")
-                    tmp = fhandle.read(1500)
-                    fhandle.close()
-                except IOError:
-                    continue
-                bad = 0
-                for bit in xrange(len(tmp)):
-                    val = ord(tmp[bit])
-                    if (val < 8) or (val > 13 and val < 32) or (val > 255):
-                        bad = bad + 1
-                if not len(tmp) or (float(bad)/float(len(tmp))) < 0.1:
-                    good.append(path)
+            good.append(path)
     return good
 
 def GetFileModTime(file_name):
@@ -343,7 +297,7 @@ def GetFileReader(file_name, enc='utf-8'):
     supplied file name. It returns a file reader using the encoding
     (enc) which defaults to utf-8. If lookup of the reader fails on
     the host system it will return an ascii reader.
-    If there is an error in creating the file reader the function 
+    If there is an error in creating the file reader the function
     will return a negative number.
     @param file_name: name of file to get a reader for
     @keyword enc: encoding to use for reading the file
@@ -355,9 +309,10 @@ def GetFileReader(file_name, enc='utf-8'):
     except (IOError, OSError):
         dev_tool.DEBUGP("[file_reader] Failed to open file %s" % file_name)
         return -1
+
     try:
-        reader = codecs.lookup(enc)[2](file_h)
-    except (LookupError, IndexError):
+        reader = codecs.getreader(enc)(file_h)
+    except (LookupError, IndexError, ValueError):
         dev_tool.DEBUGP('[file_reader] Failed to get %s Reader' % enc)
         reader = file_h
     return reader
@@ -365,9 +320,9 @@ def GetFileReader(file_name, enc='utf-8'):
 def GetFileWriter(file_name, enc='utf-8'):
     """Returns a file stream writer object for reading the
     supplied file name. It returns a file writer in the supplied
-    encoding if the host system supports it other wise it will return 
+    encoding if the host system supports it other wise it will return
     an ascii reader. The default will try and return a utf-8 reader.
-    If there is an error in creating the file reader the function 
+    If there is an error in creating the file reader the function
     will return a negative number.
     @param file_name: path of file to get writer for
     @keyword enc: encoding to write text to file with
@@ -376,41 +331,37 @@ def GetFileWriter(file_name, enc='utf-8'):
     try:
         file_h = file(file_name, "wb")
     except IOError:
-        dev_tool.DEBUGP("[file_writer] Failed to open file %s" % file_name)
+        dev_tool.DEBUGP("[file_writer][err] Failed to open file %s" % file_name)
         return -1
     try:
-        writer = codecs.lookup(enc)[3](file_h)
-    except (LookupError, IndexError):
-        dev_tool.DEBUGP('[file_writer] Failed to get %s Writer' % enc)
+        writer = codecs.getwriter(enc)(file_h)
+    except (LookupError, IndexError, ValueError):
+        dev_tool.DEBUGP('[file_writer][err] Failed to get %s Writer' % enc)
         writer = file_h
     return writer
 
-def GetPathChar():
-    """Returns the path character for the OS running the program
-    @return: character used as file path separator
-    """
-    if wx.Platform == '__WXMSW__':
-        return u"\\"
-    else:
-        return u"/"
-
 def GetUniqueName(path, name):
     """Make a file name that will be unique in case a file of the
     same name already exists at that path.
-    @param path: Root path to download folder
+    @param path: Root path to folder of files destination
     @param name: desired file name base
     @return: string
 
     """
     tmpname = os.path.join(path, name)
     if os.path.exists(tmpname):
-        ext = name.split('.')[-1]
-        fbase = name[:-1 * len(ext) - 1]
+        if '.' not in name:
+            ext = ''
+            fbase = name
+        else:
+            ext = '.' + name.split('.')[-1]
+            fbase = name[:-1 * len(ext)]
+
         inc = len([x for x in os.listdir(path) if x.startswith(fbase)])
-        tmpname = os.path.join(path, "%s-%d.%s" % (fbase, inc, ext))
+        tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
         while os.path.exists(tmpname):
             inc = inc + 1
-            tmpname = os.path.join(path, "%s-%d.%s" % (fbase, inc, ext))
+            tmpname = os.path.join(path, "%s-%d%s" % (fbase, inc, ext))
 
     return tmpname
 
@@ -419,27 +370,93 @@ def GetPathName(path):
     @param path: full path to get base of
 
     """
-    pieces = os.path.split(path)
-    return pieces[0]
+    return os.path.split(path)[0]
+
+def GetFileManagerCmd():
+    """Get the file manager open command for the current os. Under linux
+    it will check for nautilus and konqueror and return which one it finds
+    first or 'nautilus' (Gnome) if it finds neither.
+    @return: string
+
+    """
+    if wx.Platform == '__WXMAC__':
+        return 'open'
+    elif wx.Platform == '__WXMSW__':
+        return 'explorer'
+    else:
+        # Check for common linux filemanagers returning first one found
+        #          Gnome/ubuntu KDE/kubuntu  xubuntu
+        for cmd in ('nautilus', 'konqueror', 'Thunar'):
+            result = os.system("which %s > /dev/null" % cmd)
+            if result == 0:
+                return cmd
+        else:
+            return 'nautilus'
 
 def GetFileName(path):
     """Gets last atom on end of string as filename
     @param path: full path to get filename from
 
     """
-    pieces = os.path.split(path)
-    filename = pieces[-1]
-    return filename
+    return os.path.split(path)[-1]
+
+def GetFileSize(path):
+    """Get the size of the file at a given path
+    @param path: Path to file
+    @return: long
+
+    """
+    try:
+        return os.stat(path)[stat.ST_SIZE]
+    except:
+        return 0
 
 def GetExtension(file_str):
-    """Gets last atom at end of string as extension if 
+    """Gets last atom at end of string as extension if
     no extension whole string is returned
     @param file_str: path or file name to get extension from
 
     """
-    pieces = file_str.split('.')
-    extension = pieces[-1]
-    return extension
+    return file_str.split('.')[-1]
+
+def MakeNewFile(path, name):
+    """Make a new file at the given path with the given name.
+    If the file already exists, the given name will be changed to
+    a unique name in the form of name + -NUMBER + .extension
+    @param path: path to directory to create file in
+    @param name: desired name of file
+    @return: Tuple of (success?, Path of new file OR Error message)
+
+    """
+    if not os.path.isdir(path):
+        path = os.path.dirname(path)
+    fname = GetUniqueName(path, name)
+
+    try:
+        open(fname, 'w').close()
+    except (IOError, OSError), msg:
+        return (False, str(msg))
+
+    return (True, fname)
+
+def MakeNewFolder(path, name):
+    """Make a new folder at the given path with the given name.
+    If the folder already exists, the given name will be changed to
+    a unique name in the form of name + -NUMBER.
+    @param path: path to create folder on
+    @param name: desired name for folder
+    @return: Tuple of (success?, new dirname OR Error message)
+
+    """
+    if not os.path.isdir(path):
+        path = os.path.dirname(path)
+    folder = GetUniqueName(path, name)
+    try:
+        os.mkdir(folder)
+    except (OSError, IOError), msg:
+        return (False, str(msg))
+
+    return (True, folder)
 
 def ResolvAbsPath(rel_path):
     """Takes a relative path and converts it to an
@@ -447,9 +464,8 @@ def ResolvAbsPath(rel_path):
     @param rel_path: path to construct absolute path for
 
     """
-    path_char = GetPathChar()
     cwd = os.getcwd()
-    pieces = rel_path.split(path_char)
+    pieces = rel_path.split(os.sep)
     cut = 0
 
     for token in pieces:
@@ -457,52 +473,49 @@ def ResolvAbsPath(rel_path):
             cut += 1
 
         if cut > 0:
-            rpath = path_char.join(pieces[cut:])
+            rpath = os.sep.join(pieces[cut:])
             cut *= -1
-            cwd = cwd.split(path_char)
-            apath = path_char.join(cwd[0:cut])
+            cwd = cwd.split(os.sep)
+            apath = os.sep.join(cwd[0:cut])
         else:
             rpath = rel_path
             apath = cwd
 
-    return apath + path_char + rpath
+    return apath + os.sep + rpath
 
 def HasConfigDir(loc=u""):
-    """ Checks if the user has a config directory and returns True 
+    """ Checks if the user has a config directory and returns True
     if the config directory exists or False if it does not.
     @return: whether config dir in question exists on an expected path
 
     """
-    pchar = GetPathChar()
-    if os.path.exists(u"%s%s.%s%s%s" % (wx.GetHomeDir(), pchar,
-                                        ed_glob.PROG_NAME, pchar, loc)):
+    if os.path.exists(u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
+                                        ed_glob.PROG_NAME, os.sep, loc)):
         return True
     else:
         return False
 
 def MakeConfigDir(name):
-    """Makes a user config direcotry
+    """Makes a user config directory
     @param name: name of config directory to make in user config dir
 
     """
-    config_dir = wx.GetHomeDir() + GetPathChar() + u"." + ed_glob.PROG_NAME
+    config_dir = wx.GetHomeDir() + os.sep + u"." + ed_glob.PROG_NAME
     try:
-        os.mkdir(config_dir + GetPathChar() + name)
+        os.mkdir(config_dir + os.sep + name)
     except (OSError, IOError):
         pass
 
 def CreateConfigDir():
-    """ Creates the user config directory its default sub 
+    """ Creates the user config directory its default sub
     directories and any of the default config files.
     @postcondition: all default configuration files/folders are created
 
     """
     #---- Resolve Paths ----#
-    pchar = GetPathChar()
-    prof_dir = ed_glob.CONFIG['PROFILE_DIR']
-    config_dir = u"%s%s.%s" % (wx.GetHomeDir(), pchar, ed_glob.PROG_NAME)
-    profile_dir = u"%s%sprofiles" % (config_dir, pchar)
-    dest_file = u"%s%sdefault.ppb" % (profile_dir, pchar)
+    config_dir = u"%s%s.%s" % (wx.GetHomeDir(), os.sep, ed_glob.PROG_NAME)
+    profile_dir = u"%s%sprofiles" % (config_dir, os.sep)
+    dest_file = u"%s%sdefault.ppb" % (profile_dir, os.sep)
     ext_cfg = ["cache", "styles", "plugins"]
 
     #---- Create Directories ----#
@@ -523,9 +536,9 @@ def CreateConfigDir():
 
 def ResolvConfigDir(config_dir, sys_only=False):
     """Checks for a user config directory and if it is not
-    found it then resolves the absolute path of the executables 
-    directory from the relative execution path. This is then used 
-    to find the location of the specified directory as it relates 
+    found it then resolves the absolute path of the executables
+    directory from the relative execution path. This is then used
+    to find the location of the specified directory as it relates
     to the executable directory, and returns that path as a
     string.
     @param config_dir: name of config directory to resolve
@@ -534,21 +547,20 @@ def ResolvConfigDir(config_dir, sys_only=False):
            the code has proven itself.
 
     """
-    path_char = GetPathChar()
     if not sys_only:
         # Try to look for a user dir
-        user_config = u"%s%s.%s%s%s" % (wx.GetHomeDir(), path_char,
-                                        ed_glob.PROG_NAME, path_char, 
+        user_config = u"%s%s.%s%s%s" % (wx.GetHomeDir(), os.sep,
+                                        ed_glob.PROG_NAME, os.sep,
                                         config_dir)
         if os.path.exists(user_config):
-            return user_config + path_char
+            return user_config + os.sep
 
     # The following lines are used only when Editra is being run as a
     # source package. If the found path does not exist then Editra is
     # running as as a built package.
     path = __file__
-    path = path_char.join(path.split(path_char)[:-2])
-    path =  path + path_char + config_dir + path_char
+    path = os.sep.join(path.split(os.sep)[:-2])
+    path =  path + os.sep + config_dir + os.sep
     if os.path.exists(path):
         return path
 
@@ -561,41 +573,41 @@ def ResolvConfigDir(config_dir, sys_only=False):
         path = os.path.realpath(path)
 
     # Tokenize path
-    pieces = path.split(path_char)
+    pieces = path.split(os.sep)
 
     if os.sys.platform == 'win32':
         # On Windows the exe is in same dir as config directories
-        pro_path = path_char.join(pieces[:-1])
+        pro_path = os.sep.join(pieces[:-1])
 
         if os.path.isabs(pro_path):
             pass
         elif pro_path == "":
             pro_path = os.getcwd()
-            pieces = pro_path.split(path_char)
-            pro_path = path_char.join(pieces[:-1])
+            pieces = pro_path.split(os.sep)
+            pro_path = os.sep.join(pieces[:-1])
         else:
             pro_path = ResolvAbsPath(pro_path)
     else:
-        pro_path = path_char.join(pieces[:-2])
+        pro_path = os.sep.join(pieces[:-2])
 
-        if pro_path.startswith(path_char):
+        if pro_path.startswith(os.sep):
             pass
         elif pro_path == "":
             pro_path = os.getcwd()
-            pieces = pro_path.split(path_char)
+            pieces = pro_path.split(os.sep)
             if pieces[-1] not in [ed_glob.PROG_NAME.lower(), ed_glob.PROG_NAME]:
-                pro_path = path_char.join(pieces[:-1])
+                pro_path = os.sep.join(pieces[:-1])
         else:
             pro_path = ResolvAbsPath(pro_path)
 
     if os.sys.platform == "darwin":
         # On OS X the config directories are in the applet under Resources
-        pro_path = u"%s%sResources%s%s%s" % (pro_path, path_char, path_char,
-                                             config_dir, path_char)
+        pro_path = u"%s%sResources%s%s%s" % (pro_path, os.sep, os.sep,
+                                             config_dir, os.sep)
     else:
-        pro_path = pro_path + path_char + config_dir + path_char
+        pro_path = pro_path + os.sep + config_dir + os.sep
 
-    return os.path.normpath(pro_path) + path_char
+    return os.path.normpath(pro_path) + os.sep
 
 def GetResources(resource):
     """Returns a list of resource directories from a given toplevel config dir
@@ -611,7 +623,8 @@ def GetResources(resource):
     else:
         return -1
 
-def GetResourceFiles(resource, trim=True, get_all=False):
+def GetResourceFiles(resource, trim=True, get_all=False,
+                     suffix=None, title=True):
     """Gets a list of resource files from a directory and trims the
     file extentions from the names if trim is set to True (default).
     If the get_all parameter is set to True the function will return
@@ -619,10 +632,11 @@ def GetResourceFiles(resource, trim=True, get_all=False):
     files and combining them, the default behavior returns the user
     level files if they exist or the system level files if the
     user ones do not exist.
-    @param resource: name of config directory
+    @param resource: name of config directory to look in (i.e cache)
     @keyword trim: trim file extensions or not
     @keyword get_all: get a set of both system/user files or just user level
-    
+    @keyword suffix: Get files that have the specified suffix or all (default)
+    @keyword title: Titlize the results
 
     """
     rec_dir = ResolvConfigDir(resource)
@@ -639,11 +653,24 @@ def GetResourceFiles(resource, trim=True, get_all=False):
         for rec in recs:
             if os.path.isfile(rec_dir + rec) or \
               (get_all and os.path.isfile(rec_dir2 + rec)):
-                # Trim the last part of an exetion if one exists
+
+                # If a suffix was specified only keep files that match
+                if suffix is not None:
+                    if not rec.endswith(suffix):
+                        continue
+
+                # Trim the last part of an extension if one exists
                 if trim:
-                    rec = ".".join(rec.split(u".")[:-1])
-                rec_list.append(rec.title())
+                    rec = ".".join(rec.split(u".")[:-1]).strip()
+
+                # Make the resource name a title if requested
+                if title and len(rec):
+                    rec = rec[0].upper() + rec[1:]
+
+                if len(rec):
+                    rec_list.append(rec)
         rec_list.sort()
+        
         return list(set(rec_list))
 
 def Log(msg):
@@ -653,6 +680,32 @@ def Log(msg):
     """
     wx.GetApp().GetLog()(msg)
 
+def GetProxyOpener(proxy_set):
+    """Get a urlopener for use with a proxy
+    @param proxy_set: proxy settings to use
+
+    """
+    Log("[util][info] Making proxy opener with %s" % str(proxy_set))
+    proxy_info = dict(proxy_set)
+    auth_str = "%(uname)s:%(passwd)s@%(url)s"
+    url = proxy_info['url']
+    if url.startswith('http://'):
+        auth_str = "http://" + auth_str
+        proxy_info['url'] = url.replace('http://', '')
+    else:
+        pass
+
+    if len(proxy_info.get('port', '')):
+        auth_str = auth_str + ":%(port)s"
+
+    proxy_info['passwd'] = ed_crypt.Decrypt(proxy_info['passwd'],
+                                            proxy_info['pid'])
+    Log("[util][info] Formatted proxy request: %s" % \
+        (auth_str.replace('%(passwd)s', '****') % proxy_info))
+    proxy = urllib2.ProxyHandler({"http" : auth_str % proxy_info})
+    opener = urllib2.build_opener(proxy, urllib2.HTTPHandler)
+    return opener
+
 #---- GUI helper functions ----#
 def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
     """ Brighten/Darken input colour by percent and adjust alpha
@@ -661,26 +714,27 @@ def AdjustColour(color, percent, alpha=wx.ALPHA_OPAQUE):
     @type color: wx.Color
     @param percent: percent to adjust +(brighten) or -(darken)
     @type percent: int
-    @keyword alpha: amount to adjust alpha channel
+    @keyword alpha: Value to adjust alpha channel to
 
-    """ 
-    end_color = wx.WHITE
-    rdif = end_color.Red() - color.Red()
-    gdif = end_color.Green() - color.Green()
-    bdif = end_color.Blue() - color.Blue()
-    high = 100
+    """
+    radj, gadj, badj = [ int(val * (abs(percent) / 100.))
+                         for val in color.Get() ]
 
-    # We take the percent way of the color from color -. white
-    red = color.Red() + ((percent * rdif) / high)
-    green = color.Green() + ((percent * gdif) / high)
-    blue = color.Blue() + ((percent * bdif) / high)
-    return wx.Colour(max(red, 0), max(green, 0), max(blue, 0), alpha)
+    if percent < 0:
+        radj, gadj, badj = [ val * -1 for val in [radj, gadj, badj] ]
+    else:
+        radj, gadj, badj = [ val or percent for val in [radj, gadj, badj] ]
+
+    red = min(color.Red() + radj, 255)
+    green = min(color.Green() + gadj, 255)
+    blue = min(color.Blue() + badj, 255)
+    return wx.Colour(red, green, blue, alpha)
 
 def HexToRGB(hex_str):
     """Returns a list of red/green/blue values from a
     hex string.
     @param hex_str: hex string to convert to rgb
-    
+
     """
     hexval = hex_str
     if hexval[0] == u"#":
@@ -738,7 +792,7 @@ class IntValidator(wx.PyValidator):
         @param win: window to validate
 
         """
-        val = win.GetValue()      
+        val = win.GetValue()
         return val.isdigit()
 
     def OnChar(self, event):
