@@ -2,7 +2,7 @@
 # Purpose:      Presenter part
 # Author:       Roman Rolinsky <rolinsky@femagsoft.com>
 # Created:      07.06.2007
-# RCS-ID:       $Id: presenter.py 50188 2007-11-23 17:36:50Z ROL $
+# RCS-ID:       $Id: presenter.py 53700 2008-05-22 15:59:07Z ROL $
 
 import os,tempfile,shutil
 from xml.parsers import expat
@@ -67,10 +67,10 @@ class _Presenter:
         try:
             tmpFile,tmpName = tempfile.mkstemp(prefix='xrced-')
             os.close(tmpFile)
-            TRACE('Saving temporaty file: %s', tmpName)
+            TRACE('Saving temporary file: %s', tmpName)
             self.saveXML(tmpName)
-            TRACE('moving to main: %s', path)
-            shutil.move(tmpName, path)
+            TRACE('copying to the main file: %s', path)
+            shutil.copy(tmpName, path)
             self.path = path
             self.setModified(False)
         except:
@@ -372,6 +372,7 @@ class _Presenter:
         self.setApplied()
         self.unselect()
         self.setModified()
+        return node
 
     def deleteMany(self, items):
         '''Delete selected object(s).'''
@@ -390,7 +391,7 @@ class _Presenter:
 
     def cut(self):
         self.copy()
-        self.delete(view.tree.GetSelection())
+        return self.delete(view.tree.GetSelection())
 
     def copy(self):
         # Update values from panel first
@@ -579,10 +580,12 @@ class _Presenter:
         elem.setAttribute('name', STD_NAME)
         Model.setTestElem(elem)
         Model.saveTestMemoryFile()
-        xmlFlags = xrc.XRC_NO_SUBCLASSING
+        xmlFlags = 0
+        if not g.conf.useSubclassing:
+            xmlFlags |= xrc.XRC_NO_SUBCLASSING
         # Use translations if encoding is not specified
         if not Model.dom.encoding:
-            xmlFlags != xrc.XRC_USE_LOCALE
+            xmlFlags |= xrc.XRC_USE_LOCALE
         res = xrc.EmptyXmlResource(xmlFlags)
         xrc.XmlResource.Set(res)        # set as global
         # Init other handlers
@@ -594,13 +597,16 @@ class _Presenter:
         try:
             try:
                 frame, object = comp.makeTestWin(res, name)
-                if not object: return None
+                if not object:  # skip the rest
+                    raise EOFError
                 # Reset previous tree item and locate tool
                 if testWin.item:
                     view.tree.SetItemBold(testWin.item, False)
                 testWin.SetView(frame, object, item)
                 testWin.Show()
                 view.tree.SetItemBold(item, True)
+            except EOFError:
+                pass
             except NotImplementedError:
                 wx.LogError('Test window not implemented for %s' % node.getAttribute('class'))
             except:
@@ -612,7 +618,6 @@ class _Presenter:
             res.Unload(TEST_FILE)
             xrc.XmlResource.Set(None)
             wx.MemoryFSHandler.RemoveFile(TEST_FILE)
-        return object
 
     def closeTestWin(self):
         TRACE('closeTestWin')
@@ -632,7 +637,7 @@ class _Presenter:
         # Dumb refresh
         self.createTestWin(view.testWin.item)
         self.highlight(self.item)
-        if view.frame.miniFrame:
+        if view.frame.miniFrame and view.frame.miniFrame.IsShown():
             view.frame.miniFrame.Raise()
         else:
             view.frame.Raise()
@@ -657,12 +662,10 @@ class _Presenter:
 
     def generatePython(self, dataFile, pypath, embed, genGettext):
         try:
-            try:
-                import pywxrc               # !!! only for testing new pywxrc
-            except ImportError:
-                import wx.tools.pywxrc as pywxrc
+            from wx.tools import pywxrc
             rescomp = pywxrc.XmlResourceCompiler()
-            rescomp.MakePythonModule([dataFile], pypath, embed, genGettext)
+            rescomp.MakePythonModule([dataFile], pypath, embed, genGettext, 
+                                     assignVariables=False)
         except:
             logger.exception('error generating python code')
             wx.LogError('Error generating python code : %s' % pypath)
